import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';
import 'package:unify_api_example/common/index.dart';
import 'package:unify_api_example/utils/index.dart';

class CartExample extends StatefulWidget {
  final UnifyClient client;

  const CartExample({Key key, this.client}) : super(key: key);

  @override
  _CartExampleState createState() => _CartExampleState();
}

class _CartExampleState extends State<CartExample> {
  CartOrder _details;
  @override
  void initState() {
    widget.client.getCartOrder().then((details) {
      setState(() {
        _details = details;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Cart Order"),
        actions: <Widget>[
          JsonButton(json: _details?.json),
        ],
      ),
      body: Builder(
        builder: (context) {
          if (_details == null)
            return Center(child: CircularProgressIndicator());
          return ListView(
            children: <Widget>[
              ListTile(
                title: Text("Order Number"),
                subtitle: Text(_details.orderNumber),
              ),
              ListTile(
                title: Text("Order Total"),
                subtitle: Text(_details.orderTotal.toString()),
              ),
              ListTile(
                title: Text("Order Status"),
                subtitle: Text(_details.orderStatus),
              ),
              ListTile(
                title: Text("Shipping Address"),
                subtitle: Text(_details.shippingAddress),
              ),
              ListTile(
                title: Text("Billing Address"),
                subtitle: Text(_details.billingAddress),
              ),
              ListTile(
                title: Text("Date Created"),
                subtitle: Text(_details.dateCreated.toString()),
              ),
              ListTile(
                title: Text("Date Modified"),
                subtitle: Text(_details.dateModified.toString()),
              ),
              ListTile(
                title: Text("Cart Items"),
                subtitle: Text("Tap To View"),
                onTap: () {
                  var items = _details?.items ?? [];

                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.name),
                            subtitle: Text(i.quantity.toString()),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
            ],
          );
        },
      ),
    );
  }
}
