import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:flutter/foundation.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart';

import '../../classes/enum_list.dart';
import '../../services/database/database.dart';
import '../../utils/utils.dart';
import '../bloc.dart';
import 'bloc.dart';

class AlertsBloc extends Bloc<AlertsEvent, AlertsState> {
  @override
  AlertsState get initialState => InititalAlertsState();

  @override
  Stream<AlertsState> mapEventToState(AlertsEvent event) async* {
    if (event is AlertsCheck) {
      for (var folder in DashboardAlertType.values) {
        yield AlertsLoadingState(folder);
        final _folder = folder;
        yield* _refreshFolder(_folder, event.user, checkStale: true);
      }
      yield AlertsReadyState();
    }
    if (event is RefreshAlerts) {
      yield AlertsLoadingState(event.folder);
      final _folder = event.folder;
      yield* _refreshFolder(_folder, event.info);
      final _context = event.context;
      event.scaffold
          .dispatch(ShowSnakBar('refreshed ${describeEnum(_folder)} alerts', _context));
      yield AlertsReadyState();
    }
  }

  Stream<AlertsState> _refreshFolder(
      DashboardAlertType _folder, AccountInfo user,
      {bool checkStale = false}) async* {
    switch (_folder) {
      case DashboardAlertType.customer:
        yield* _checkStale(checkStale, user, _folder);
        break;
      case DashboardAlertType.hotProspect:
        yield* _checkStale(checkStale, user, _folder);
        break;
      case DashboardAlertType.partners:
        yield* _checkStale(checkStale, user, _folder);
        break;
      case DashboardAlertType.pipeline:
        yield* _checkStale(checkStale, user, _folder);
        break;
      case DashboardAlertType.opportunity:
        yield* _checkStale(checkStale, user, _folder);
        break;
      case DashboardAlertType.all:
        break;
    }
  }

  Stream<AlertsState> _checkStale(
      bool checkStale, AccountInfo user, DashboardAlertType _folder) async* {
    if (checkStale) {
      final _item = await GetIt.instance<DatabaseService>()
          .dashboardDao
          .getLastAlert(user.id, _folder);
      if (_item == null) {
        yield* _updateAlerts(kDefaultRows, user, _folder);
      }
    } else {
      yield* _updateAlerts(kDefaultRows, user, _folder);
    }
  }

  Stream<AlertsState> _updateAlerts(
      int _rows, AccountInfo user, DashboardAlertType folder) async* {
    var _empty = false;
    var _page = 1;
    while (!_empty) {
      switch (folder) {
        case DashboardAlertType.customer:
          final _list =
              await user.client.listCustomerAlerts(page: _page, rows: _rows);
          if (_list != null && _list.isNotEmpty) {
            await GetIt.instance<DatabaseService>()
                .dashboardDao
                .batchUpdate(user.id, _list, folder: folder);
          } else {
            _empty = true;
          }
          _page++;
          break;
        case DashboardAlertType.hotProspect:
          final _list =
              await user.client.listHotProspectAlerts(page: _page, rows: _rows);
          if (_list != null && _list.isNotEmpty) {
            await GetIt.instance<DatabaseService>()
                .dashboardDao
                .batchUpdate(user.id, _list, folder: folder);
          } else {
            _empty = true;
          }
          _page++;
          break;
        case DashboardAlertType.partners:
          final _list =
              await user.client.listDashboardPartners(page: _page, rows: _rows);
          if (_list != null && _list.isNotEmpty) {
            await GetIt.instance<DatabaseService>().dashboardDao.batchUpdate(
                user.id, null,
                folder: folder, partnerAlerts: _list);
          } else {
            _empty = true;
          }
          _page++;
          break;
        case DashboardAlertType.pipeline:
          final _list =
              await user.client.listPipelineAlerts(page: _page, rows: _rows);
          if (_list != null && _list.isNotEmpty) {
            await GetIt.instance<DatabaseService>()
                .dashboardDao
                .batchUpdate(user.id, _list, folder: folder);
          } else {
            _empty = true;
          }
          _page++;
          break;
        case DashboardAlertType.opportunity:
          final _list = await user.client
              .listOpportunitiyAlerts(page: _page, rows: _rows);
          if (_list != null && _list.isNotEmpty) {
            await GetIt.instance<DatabaseService>()
                .dashboardDao
                .batchUpdate(user.id, _list, folder: folder);
          } else {
            _empty = true;
          }
          _page++;
          break;
        case DashboardAlertType.all:
          break;
      }
    }
    yield AlertsReadyState();
  }
}
