import 'dart:convert';

import 'package:bloc/bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart';

import '../../services/database/database.dart';
import '../../utils/utils.dart';
import '../bloc.dart';
import 'bloc.dart';

class LeadsBloc extends Bloc<LeadsEvent, LeadsState> {
  @override
  LeadsState get initialState => InitialLeadsState();

  @override
  Stream<LeadsState> mapEventToState(
    LeadsEvent event,
  ) async* {
    if (event is LeadsCheck) {
      yield LeadsLoadingState();
      const _rows = kDefaultRows;
      final _item = await GetIt.instance<DatabaseService>()
          .leadsDao
          .getLastLead(event.user.id);
      if (_item == null) {
        yield* _updateLeads(_rows, event.user);
      }
      yield LeadsReadyState();
    }
    if (event is RefreshLeads) {
      yield LeadsLoadingState();
      const _row = kDefaultRows;
      yield* _updateLeads(_row, event.user);
      event.scaffold.dispatch(ShowSnakBar('refreshed leads', event.context));
      yield LeadsReadyState();
    }
    if (event is UpdateGroupItems) {
      yield* _updateGroupItems(event);
    }
    if (event is UpdateLeadDetails) {
      yield* _updateDetails(event);
    }
  }
}

Stream<LeadsState> _updateDetails(UpdateLeadDetails event) async* {
  final leadDetails = await event.user.client.getLeadDetails(event.leadId);
  await GetIt.instance<DatabaseService>()
      .leadsDao
      .insertDetails(event.userId, event.leadId, json.encode(leadDetails.json));

  yield LeadsReadyState();
}

Stream<LeadsState> _updateGroupItems(UpdateGroupItems event) async* {
  final user = event.user;
  final group = event.group;
  if (group.count > 0) {
    final _groupLeads = await user.client
        .listLeadsForLeadGroup(group.groupId, rows: 1000, page: 1);
    if (_groupLeads != null && _groupLeads.isNotEmpty) {
      await GetIt.instance<DatabaseService>().groupItemDao.batchUpdateLead(
          group.groupName, group.groupId, user.id, _groupLeads);
    }
  }
  yield LeadsReadyState();
}

Stream<LeadsState> _updateLeads(int _rows, AccountInfo user) async* {
  var _empty = false;
  var _page = 1;

  final _leadGroups = await user.client.listLeadGroups();
  if (_leadGroups != null && _leadGroups.isNotEmpty) {
    await GetIt.instance<DatabaseService>()
        .leadGroupDao
        .batchUpdate(user.id, _leadGroups);
  }
  // for (var group in _leadGroups) {
  //   if (group.count > 0) {
  //     final _groupLeads = await user.client
  //         .listLeadsForLeadGroup(group.id, rows: _rows, page: 1);
  //     if (_groupLeads != null && _groupLeads.isNotEmpty) {
  //       await GetIt.instance<DatabaseService>()
  //           .groupItemDao
  //           .batchUpdateLead(group.name, group.id, user.id, _groupLeads);
  //     }
  //   }
  // }

  while (!_empty) {
    final _leads = await user.client.listLeads(
      page: _page,
      rows: _rows,
    );
    if (_leads != null && _leads.isNotEmpty) {
      await GetIt.instance<DatabaseService>()
          .leadsDao
          .batchUpdate(user.id, _leads);
    } else {
      _empty = true;
    }
    _page++;
  }
  yield LeadsReadyState();
}
