import 'package:moor/moor.dart';

import '../database.dart';
import '../tables/favorites.dart';

part 'favorites.g.dart';

@UseDao(
  tables: [FavoriteRecords],
  queries: {
    'favoriteByType':
      'SELECT * FROM favorite WHERE user_id = :userId AND type = :type ORDER BY type ASC',
    'favoriteById':
      'SELECT * FROM favorite WHERE user_id = :userId AND favorite_id = :favoriteId LIMIT 1',
    'favoriteByTypeId':
      'SELECT * FROM favorite WHERE user_id = :userId AND type_id = :typeId LIMIT 1',
    'favoritesByUser':
      'SELECT * FROM favorite WHERE user_id = :userID ORDER BY type ASC'
  }
)
class FavoritesDao extends DatabaseAccessor<DatabaseService>
    with _$FavoritesDaoMixin {
  FavoritesDao(DatabaseService db) : super(db);

  Future addItem(String type, String typeId, String userId) async {
    final _item = FavoriteRecordsCompanion(
      userId: addField<String>(userId),
      type: addField<String>(type),
      typeId: addField<String>(typeId),
    );
    return await transaction(() async {
      await into(favoriteRecords).insert(_item, orReplace: true);
    });
  }

  Future removeItem(int favoriteId, String userId) async {
    return await transaction(() async {
      return (delete(favoriteRecords)
            ..where((l) => l.userId.equals(userId))
            ..where((l) => l.favoriteId.equals(favoriteId)))
          .go();
    });
  }
}