import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/lead_group.dart';

part 'lead_group.g.dart';

@UseDao(
  tables: [LeadGroupRecords],
  queries: {
    'groupsByUser':
        'SELECT * FROM leadGroup WHERE user_id = :userId ORDER BY last_updated ASC'
  }
)
class LeadGroupDao extends DatabaseAccessor<DatabaseService> with _$LeadGroupDaoMixin {
  LeadGroupDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.LeadGroup> groups) async {
    final _items = <LeadGroupRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var group in groups){
      final _item = LeadGroupRecordsCompanion(
        userId: addField<String>(userId),
        groupName: addField<String>(group.name),
        groupId: addField<String>(group.id),
        count: addField<int>(group.count),
        lastUpdated: addField<DateTime>(_updated)
      );
      _items.add(_item);
    }
    await into(leadGroupRecords).insertAll(_items, orReplace: true);
    await cleanGroups(userId, _updated);
  }

  Future cleanGroups(String userId, DateTime lastUpdated) async {
    return await (delete(leadGroupRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  Future addItem(String groupName, String userId, int count) async {
    final _item = LeadGroupRecordsCompanion(
      groupName: addField<String>(groupName),
      userId: addField<String>(userId),
      count: addField<int>(count),
    );
    return await transaction(() async {
      await into(leadGroupRecords).insert(_item, orReplace: true);
    });
  }

  Future removeItem(String groupId, String userId) async {
    return await transaction(() async {
      return (delete(leadGroupRecords)
            ..where((l) => l.userId.equals(userId))
            ..where((l) => l.groupId.equals(groupId)))
          .go();
    });
  }
}