import 'package:flutter/foundation.dart';
import 'package:moor/moor.dart';

import '../database.dart';
import '../tables/rates.dart';

part 'rate.g.dart';

@UseDao(
    tables: [RatesRecords],
    queries: {'rateByType': 'SELECT * FROM rates WHERE type = :type Limit 1'})
class RateDao extends DatabaseAccessor<DatabaseService> with _$RateDaoMixin {
  RateDao(DatabaseService db) : super(db);

  Future updateRates(String rates, double currentRate, RateType type) async {
    final now = DateTime.now();
    final _rate = await rateByType(describeEnum(type));
    if (_rate != null && _rate.isNotEmpty) {
      return await (update(ratesRecords)
            ..where((t) => t.type.equals(describeEnum(type))))
          .write(
        _rate.first
            .copyWith(rates: rates, currentRate: currentRate, lastUpdated: now),
      );
    }
    return await into(ratesRecords).insert(
      RatesRecordsCompanion(
          type: addField<String>(describeEnum(type)),
          rates: addField<String>(rates),
          currentRate: addField<double>(currentRate),
          lastUpdated: addField<DateTime>(now)),
      orReplace: true,
    );
  }
}
