import 'package:example_db/data/services/database/database.dart';
import 'package:example_db/data/services/database/tables/video_folder.dart';
import 'package:moor/moor.dart';

part 'video_folders.g.dart';

@UseDao(tables: [
  VideoFolderRecords
], queries: {
  'archivedVideoById':
      'SELECT * FROM videoFolder WHERE user_id = :useId AND video_id = :videoId And archived = true ORDER BY date_modified',
  'sharedVideos':
      'SELECT * FROM videoFolder WHERE user_id = :userId AND shared = true ORDER BY date_modified',
  'archivedVideos':
      'SELECT * FROM videoFolder WHERE user_id = :userId AND archived = true ORDER BY date_modified',
  'videosById':
      'SELECT * FROM videoFolder WHERE user_id = :userId AND video_id = :videoId LIMIT 1',
  'videosByUser':
      'SELECT * FROM videoFolder WHERE user_id = :userId ORDER BY date_modified ASC',
})
class VideoFolderDao extends DatabaseAccessor<DatabaseService>
    with _$VideoFolderDaoMixin {
  VideoFolderDao(DatabaseService db) : super(db);

  Future sharedVid(String userId, String videoId, bool shared) async {
    final _item = VideoFolderRecordsCompanion(
        userId: addField<String>(userId),
        videoId: addField<String>(videoId),
        shared: addField<bool>(shared));
    return await transaction(() async {
      await into(videoFolderRecords).insert(_item, orReplace: true);
    });
  }

  Future archivedVid(String userId, String videoId, bool archived) async {
    final _item = VideoFolderRecordsCompanion(
        userId: addField<String>(userId),
        videoId: addField<String>(videoId),
        archived: addField<bool>(archived));
    return await transaction(() async {
      await into(videoFolderRecords).insert(_item, orReplace: true);
    });
  }

  Future updateItem(
      String userId, String videoId, bool archived, bool shared) async {
    final _item = await videosById(userId, videoId);

    final _video = _item.first.copyWith(
        archived: archived, shared: shared, dateModified: DateTime.now());

    return into(videoFolderRecords).insert(_video, orReplace: true);
  }

  Future removeItem(String userId, String videoId) async {
    return await transaction(() async {
      return (delete(videoFolderRecords)
            ..where((l) => l.userId.equals(userId))
            ..where((l) => l.videoId.equals(videoId)))
          .go();
    });
  }
}
