import 'package:example_db/data/blocs/auth/bloc.dart';
import 'package:example_db/data/services/database/database.dart';
import 'package:example_db/plugins/url_launcher/url_launcher.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

class AccountScreen extends StatelessWidget {
  static const String route = '/account';
  final User user;

  const AccountScreen({Key key, this.user}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    final _user = user ?? AuthBloc.currentUser(context);
    return Scaffold(
      appBar: user == null ? null : AppBar(title: Text('Account Info')),
      body: Column(
        children: <Widget>[
          Container(
            height: 200.0,
            padding: EdgeInsets.all(8.0),
            child: Image.network(
              _user.companyImage,
              fit: BoxFit.fitWidth,
            ),
          ),
          ListTile(
            leading: _user?.profileImage == null || _user.profileImage.isEmpty
                ? CircleAvatar(
                    child: Icon(Icons.person),
                  )
                : CircleAvatar(
                    backgroundImage: NetworkImage(_user.profileImage),
                  ),
            title: Text('${_user.firstName} ${_user.lastName}'),
            subtitle: Text(_user.title),
          ),
          ListTile(
            leading: Icon(Icons.email),
            title: Text('Email'),
            subtitle: Text(_user.email),
          ),
          ListTile(
            leading: Icon(Icons.phone),
            title: Text('Primary Phone'),
            subtitle: Text(_user.primaryPhone),
          ),
          ListTile(
            leading: Icon(Icons.phone),
            title: Text('Secondary Phone'),
            subtitle: Text(_user.secondaryPhone),
          ),
          if (_user?.losLicenseNumber != null)
            ListTile(
              leading: Icon(Icons.label),
              title: Text('NMLS Number'),
              subtitle: Text(_user.losLicenseNumber),
            ),
          ListTile(
            title: ButtonBar(
              mainAxisSize: MainAxisSize.min,
              alignment: MainAxisAlignment.center,
              children: <Widget>[
                IconButton(
                  icon: Icon(FontAwesomeIcons.facebook),
                  onPressed:
                      _user?.facebookUrl == null || _user.facebookUrl.isEmpty
                          ? null
                          : () => UrlUtils.open(_user.facebookUrl),
                ),
                IconButton(
                  icon: Icon(FontAwesomeIcons.twitter),
                  onPressed:
                      _user?.twitterUrl == null || _user.twitterUrl.isEmpty
                          ? null
                          : () => UrlUtils.open(_user.twitterUrl),
                ),
                IconButton(
                  icon: Icon(FontAwesomeIcons.linkedin),
                  onPressed:
                      _user?.linkedInUrl == null || _user.linkedInUrl.isEmpty
                          ? null
                          : () => UrlUtils.open(_user.linkedInUrl),
                ),
                IconButton(
                  icon: Icon(Icons.web),
                  onPressed:
                      _user?.websiteUrl == null || _user.websiteUrl.isEmpty
                          ? null
                          : () => UrlUtils.open(_user.websiteUrl),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
