import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../../data/blocs/alerts/bloc.dart';
import '../../data/blocs/auth/bloc.dart';
import '../../data/blocs/bloc.dart';
import '../../data/classes/enum_list.dart';
import '../common/common.dart';
import 'common/folder.dart';

class NotificationsScreen extends StatefulWidget {
  @override
  _NotificationsScreenState createState() => _NotificationsScreenState();
}

class _NotificationsScreenState extends State<NotificationsScreen> {
  int _currentIndex = 0;
  @override
  Widget build(BuildContext context) {
    final _folders = [
      DashboardAlertType.opportunity,
      DashboardAlertType.customer,
      DashboardAlertType.hotProspect,
      DashboardAlertType.pipeline,
      DashboardAlertType.partners,
    ];
    return DefaultTabController(
      initialIndex: _currentIndex,
      length: _folders.length,
      child: Scaffold(
        // appBar: AppBar(
        //   title: Text('Alerts'),
        //   leading: MenuButton(),
        //   actions: <Widget>[
        //     IconButton(
        //       icon: Icon(Icons.refresh),
        //       onPressed: () {
        //         setState(() {
        //           for(var folder in _folders){
        //             refresh(context, folder);
        //           }
        //         });
        //       }
        //     )
        //   ],
        //   bottom: TabBar(
        //     onTap: (val) => _currentIndex =val,
        //     isScrollable: true,
        //     tabs: <Widget>[
        //       for (var folder in _folders) ...[
        //         Tab(
        //           text: _getTitle(folder),
        //         )
        //       ]
        //     ],
        //   ),
        // ),
        appBar: PreferredSize(
          preferredSize: Size.fromHeight(60),
          child: Center(
            child: TabBar(
              onTap: (val) => _currentIndex = val,
              isScrollable: true,
              tabs: <Widget>[
                for (var folder in _folders) ...[
                  Tab(
                    text: _getTitle(folder),
                  )
                ]
              ],
            ),
          ),
        ),
        body: TabBarView(
          children: <Widget>[
            for (var folder in _folders) ...[AlertFolderScreen(folder: folder)]
          ],
        ),
      ),
    );
  }

  String _getTitle(DashboardAlertType folder) {
    switch (folder) {
      case DashboardAlertType.opportunity:
        return 'Opportunities';
      case DashboardAlertType.customer:
        return 'Customer';
      case DashboardAlertType.hotProspect:
        return 'Hot Prospect';
      case DashboardAlertType.partners:
        return 'Partners';
      case DashboardAlertType.pipeline:
        return 'My Pipeline';
      default:
    }
    return describeEnum(folder);
  }

  static refresh(BuildContext context, DashboardAlertType folder) {
    final alerts = BlocProvider.of<AlertsBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return alerts.dispatch(RefreshAlerts(
      info: _user,
      folder: folder,
      scaffold: scaffold,
      context: context
    ));
  }
}
