import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart';

import '../../data/blocs/auth/auth_bloc.dart';
import '../../data/classes/phone.dart';
import '../../data/utils/defaults.dart';
import '../../data/utils/number_format.dart';
import '../../data/utils/popUp.dart';
import '../common/app_input_field.dart';
import 'phone_tile.dart';

class ActionTileEvent extends StatefulWidget {
  ActionTileEvent({
    Key key,
    this.title,
    this.contactDetails,
    this.leadDetails,
    @required this.type,
  });

  final String title;
  final ActionType type;
  final ContactDetails contactDetails;
  final LeadDetails leadDetails;

  @override
  _ActionTileEventState createState() => _ActionTileEventState();
}

class _ActionTileEventState extends State<ActionTileEvent> {

  final TextEditingController controller = TextEditingController();
  final TextEditingController titleController = TextEditingController();
  final TextEditingController commentController = TextEditingController();

  final FocusNode mainFocus = FocusNode();
  final FocusNode titleFocus = FocusNode();
  final FocusNode commentFocus = FocusNode();

  final _formKey = GlobalKey<FormState>();

  List<String> responses = responsesContact;
  List<Phone> contactPhone = [];

  DateTime _date;
  String selection;
  bool _dataAdded = false,
      _email = false,
      _txtMsg = false,
      _addCalender = false,
      _pushNotif = false,
      _dismissChng = false;
  Duration _timer;
  int segmentType = 0, segmentTime = 0;

  Phone _personal, _external;

    

  @override
  void initState() {
    super.initState();
    selection = responses.first;

    if(widget?.leadDetails != null){
      contactPhone.add(Phone.fromJson(widget.leadDetails.cellPhone.json));
      contactPhone.add(Phone.fromJson(widget.leadDetails.homePhone.json));
      contactPhone.add(Phone.fromJson(widget.leadDetails.officePhone.json));
      contactPhone.add(Phone.fromJson(widget.leadDetails.secondaryCellPhone.json));
    }
    if(widget?.contactDetails != null){
      contactPhone.add(Phone.fromJson(widget.contactDetails.cellPhone.json));
      contactPhone.add(Phone.fromJson(widget.contactDetails.homePhone.json));
      contactPhone.add(Phone.fromJson(widget.contactDetails.officePhone.json));
      contactPhone.add(Phone.fromJson(widget.contactDetails.secondaryCellPhone.json));
    }
  }

  @override
  Widget build(BuildContext context) {
    FocusScope.of(context).requestFocus(mainFocus);
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
        actions: <Widget>[
          IconButton(
              icon: Icon(Icons.check),
              onPressed: () async {
                var pass = false;
                if (widget?.leadDetails != null) {
                  switch (widget.type) {
                    case ActionType.note:
                      await widget.leadDetails.client.addLeadNote(
                          widget.leadDetails.id, controller.value.text);
                      break;
                    case ActionType.logresponse:
                      await widget.leadDetails.client.addLeadLogResponse(
                          widget.leadDetails.id,
                          action: selection,
                          comment: controller.value.text);
                      break;
                    case ActionType.followup:
                      // TODO: Handle this case.
                      break;
                  }
                }
                if (widget?.contactDetails != null) {
                  switch (widget.type) {
                    case ActionType.note:
                      await widget.contactDetails.client.addContactNote(
                          widget.contactDetails.id, controller.value.text);
                      break;
                    case ActionType.logresponse:
                      await widget.contactDetails.client.addContactLogResponse(
                          widget.contactDetails.id,
                          action: selection,
                          comment: controller.value.text);
                      break;
                    case ActionType.followup:
                      // TODO: Handle this case.
                      break;
                  }
                }
                Navigator.pop(context, pass);
              })
        ],
      ),
      body: Container(
        child: Builder(builder: (_) {
          switch (widget.type) {
            case ActionType.note:
              return _buildNote();
            case ActionType.logresponse:
              return _buildLogResponse();
            case ActionType.followup:
              return _buildFollowUp();
          }
          return Container();
        }),
      ),
    );
  }

  Widget _buildFollowUp() {

    final _user = AuthBloc.currentInfo(context);

    

    final phones = <Phone>[];

    if (_user?.primaryPhone != null) {
      phones.add(Phone.fromString(_user?.primaryPhone.toString()));
    }
    if (_user?.secondaryPhone != null) {
      phones.add(Phone.fromString(_user?.secondaryPhone.toString()));
    }
    
    return Padding(
      padding: EdgeInsets.only(top: 20),
      child: Column(
        children: <Widget>[
          Row(
            children: <Widget>[
              Expanded(
                child: CupertinoSegmentedControl<int>(
                  children: <int, Widget>{
                    0: Text('Task', maxLines: 1),
                    1: Text('Appointment', maxLines: 1),
                    2: Text(
                        MediaQuery.of(context).size.width < kSmallestPhone
                            ? 'Call'
                            : 'Scheduled Call',
                        maxLines: 1),
                  },
                  onValueChanged: (value) {
                    if (mounted) {
                      setState(() {
                        segmentType = value;
                        if(segmentType == 2){
                          _personal = Phone.fromJson(_user.primaryPhone.json);
                          _external = contactPhone.first;
                        }
                      });
                    }
                  },
                  groupValue: segmentType,
                ),
              ),
            ],
          ),
          Form(
            key: _formKey,
            onWillPop: () => _canLeave(context),
            child: new _InfoForm(
              title: titleController,
              comments: commentController,
            ),
            onChanged: () {
              _dataAdded = true;
            },
          ),
          Divider(),
          Row(
            children: <Widget>[
              Expanded(
                child: CupertinoSegmentedControl<int>(
                  children: <int, Widget>{
                    0: Text('From Now'),
                    1: Text('Date and Time'),
                  },
                  groupValue: segmentTime,
                  onValueChanged: (value) {
                    if (mounted) {
                      setState(() {
                        segmentTime = value;
                        if (segmentTime == 1) {
                          _date = DateTime.now();
                        } else {
                          _date = null;
                        }
                      });
                    }
                  },
                ),
              )
            ],
          ),
          AnimatedCrossFade(
            firstChild: Container(
              child: ListTile(
                leading: Icon(Icons.timer),
                title: Text('Select Time'),
                subtitle: _timer == null
                    ? Text('Tap to Edit')
                    : Text(durationFormat(_timer)),
                onTap: () {
                  FocusScope.of(context).requestFocus(FocusNode());
                  showCupertinoModalPopup<void>(
                    context: context,
                    builder: (context) {
                      return _buildBottomPicker(
                        CupertinoTimerPicker(
                          mode: CupertinoTimerPickerMode.hm,
                          initialTimerDuration: _timer ?? Duration(),
                          onTimerDurationChanged: (newTimer) {
                            if (mounted) setState(() => _timer = newTimer);
                          },
                        ),
                      );
                    },
                  );
                },
              ),
            ),
            secondChild: Container(
              child: ListTile(
                leading: Icon(Icons.date_range),
                title: Text('Select Date and Time'),
                subtitle: _date == null
                    ? Text('Tap to Edit')
                    : Text(DateFormat.yMd().add_jm().format(_date)),
                onTap: () {
                  FocusScope.of(context).requestFocus(FocusNode());
                  showCupertinoModalPopup<void>(
                    context: context,
                    builder: (context) {
                      return _buildBottomPicker(
                        CupertinoDatePicker(
                          mode: CupertinoDatePickerMode.dateAndTime,
                          initialDateTime: _date ?? DateTime.now(),
                          onDateTimeChanged: (newDateTime) {
                            if (mounted) setState(() => _date = newDateTime);
                          },
                        ),
                      );
                    },
                  );
                },
              ),
            ),
            crossFadeState: segmentTime == 0
                ? CrossFadeState.showFirst
                : CrossFadeState.showSecond,
            duration: const Duration(milliseconds: 300),
          ),
          Divider(),
          ListTile(
            leading: Icon(Icons.email),
            title: Text('Email'),
            trailing: Switch.adaptive(
              value: _email,
              onChanged: (value) {
                setState(() {
                  _email = value;
                });
              },
            ),
          ),
          ListTile(
            leading: Icon(Icons.message),
            title: Text('Text Message'),
            trailing: Switch.adaptive(
                value: _txtMsg,
                onChanged: (value) {
                  setState(() {
                    _txtMsg = value;
                  });
                }),
          ),
          ListTile(
            leading: Icon(Icons.event),
            title: Text('Add to Calendar'),
            trailing: Switch.adaptive(
              value: _addCalender,
              onChanged: (value) {
                setState(() {
                  _addCalender = value;
                });
              },
            ),
          ),
          ListTile(
            leading: Icon(Icons.notification_important),
            title: Text('Push Notification'),
            trailing: Switch.adaptive(
              value: _pushNotif,
              onChanged: (value) {
                setState(() {
                  _pushNotif = value;
                });
              },
            ),
          ),
          Divider(),
          if (segmentType == 2) ...[
            phones == null || phones.isEmpty
                  ? ListTile(
                      leading: Icon(Icons.phone),
                      title: Text('Call Me At'),
                      subtitle: Text('No Phones Found'),
                    )
                  : PhonePickerTile(
                      leading: Icon(Icons.phone),
                      title: Text('Call Me At'),
                      subtitle: _personal == null
                          ? Text('Tap to Edit')
                          : Text(_personal.toString()),
                      phones: phones,
                      initialItem: _personal,
                      phoneChanged: (index) {
                        if (mounted)
                          setState(() {
                            _personal = phones[index];
                          });
                      },
                    ),
              contactPhone == null || contactPhone.isEmpty
                  ? ListTile(
                      leading: Icon(Icons.phone),
                      title: Text('Call Contact At'),
                      subtitle: Text('No Phones Found'),
                    )
                  : PhonePickerTile(
                      leading: Icon(Icons.phone),
                      title: Text('Call Contact At'),
                      subtitle: _external == null
                          ? Text('Tap to Edit')
                          : Text(_external.toString()),
                      phones: contactPhone,
                      initialItem: _external,
                      phoneChanged: (index) {
                        if (mounted)
                          setState(() {
                            _external = contactPhone[index];
                          });
                      },
                    ),
              Divider(),
          ],
          ListTile(
            leading: Icon(Icons.notifications_active),
            title: Text('Dismiss On Change'),
            trailing: Switch.adaptive(
              value: _dismissChng,
              onChanged: (value) {
                if (mounted) setState(() => _dismissChng = value);
              },
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildNote() {
    return Padding(
      padding: EdgeInsets.all(24),
      child: AppInputField(
        focus: mainFocus,
        controller: controller,
        required: true,
        multiLine: true,
        autoFocus: true,
        name: 'Note',
      ),
    );
  }

  Future<bool> _canLeave(BuildContext context) {
    if (!_dataAdded) {
      return new Future<bool>.value(true);
    } else
      return _prompt(context);
  }

  Future<bool> _prompt(BuildContext context) async {
    return await showConfirmationPopup(
      context,
      title: 'Warning - Unsaved Changes',
      detail: 'Do you want to discard all changes?',
    );
  }

  Widget _buildLogResponse() {
    return Padding(
      padding: const EdgeInsets.only(top: 20.0),
      child: Column(
        children: <Widget>[
          ListTile(
            leading: Icon(Icons.label),
            title: Text('Select a Response'),
            subtitle: DropdownButton<String>(
              value: selection,
              items: responses
                  .map(
                    (String item) => DropdownMenuItem<String>(
                        value: item,
                        child: SizedBox(
                            child: Text(
                          item,
                          textScaleFactor: 1,
                        ))),
                  )
                  .toList(),
              onChanged: (val) {
                setState(() {
                  selection = val;
                });
              },
            ),
          ),
          AppInputField(
              focus: mainFocus,
              controller: controller,
              required: true,
              multiLine: true,
              autoFocus: true,
              name: 'Comments')
        ],
      ),
    );
  }
}

Widget _buildBottomPicker(Widget picker) {
  return Theme(
    data: ThemeData.light(),
    child: Container(
      height: 216,
      padding: const EdgeInsets.only(top: 6.0),
      color: CupertinoColors.white,
      child: DefaultTextStyle(
        style: const TextStyle(
          color: CupertinoColors.black,
          fontSize: 22.0,
        ),
        child: GestureDetector(
          onTap: () {},
          child: SafeArea(
            top: false,
            child: picker,
          ),
        ),
      ),
    ),
  );
}

enum ActionType { note, logresponse, followup }

const List<String> responsesContact = [
  "Called and Spoke - Great Call",
  "Called and Spoke - General",
  "Called Left Message",
  "Called No Answer",
  "General Comment",
  "Face to Face",
  "Break Bread"
];

const List<String> responsesLead = [
  "No Contact Left Message",
  "No Contact Emailed Only",
  "No Contact Nurture",
  "Contacted Set Appointment",
  "Contacted Quoted",
  "Contacted Nurture",
  "Application Received",
  "Application Disclosures Sent",
  "Application Prequal",
  "Application in Process",
  "Inactive Borrower Not Interested",
  "Inactive DNQ Property",
  "Inactive DNQ Borrower",
  "Inactive Withdrawn",
  "Inactive DNQ Borrower",
  "Bad Lead Return To Vendor",
  "General Comment"
];

class _InfoForm extends StatelessWidget {
  const _InfoForm({
    Key key,
    @required TextEditingController title,
    @required TextEditingController comments,
  })  : _title = title,
        _comments = comments,
        super(key: key);

  final TextEditingController _title;
  final TextEditingController _comments;

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        AppInputField(
          required: true,
          leading: Icon(Icons.title),
          name: 'Title',
          controller: _title,
        ),
        AppInputField(
          leading: Icon(Icons.message),
          name: 'Comments',
          controller: _comments,
          multiLine: true,
        ),
      ],
    );
  }
}
