import 'package:flutter/material.dart';

import '../../data/utils/is_empty.dart';
import '../../data/utils/phone_email_actions.dart';

class EmailTile extends StatelessWidget {
  const EmailTile({
    this.label,
    @required this.email,
    this.icon,
    this.emailPressed,
  });

  final String email, label;
  final IconData icon;
  final VoidCallback emailPressed;

  @override
  Widget build(BuildContext context) {
    if (isNullOrEmpty(email)) {
      return ListTile(
        leading: Icon(icon ?? Icons.email),
        title: Text(
          label ?? 'Email',
          textScaleFactor: 1,
        ),
        subtitle: const Text(
          "No Email Found",
          textScaleFactor: 1,
        ),
      );
    }
    return ListTile(
      leading: Icon(icon ?? Icons.email),
      title: Text(
        label ?? 'Email',
        textScaleFactor: 1,
      ),
      subtitle: Text(
        email,
        textScaleFactor: 1,
      ),
      onTap: () {
        if (emailPressed != null) emailPressed();
        return sendEmail(context, recipients: [email]);
      },
    );
  }
}
