import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart' as api;

import '../../data/services/database/database.dart';

class ManageGroups extends StatefulWidget {
  ManageGroups(
      {Key key,
      this.leadGroups,
      this.contactGroups,
      this.userInfo,
      this.lead,
      this.contact,
      this.type,
      this.returnList = false})
      : super(key: key);

  final Lead lead;
  final Contact contact;
  final String type;
  final api.AccountInfo userInfo;
  final List<api.LeadGroup> leadGroups;
  final List<api.ContactGroup> contactGroups;
  final bool returnList;

  @override
  _ManageGroupsState createState() => _ManageGroupsState();
}

class _ManageGroupsState extends State<ManageGroups> {
  //String widget.type;

  List<api.LeadGroup> leads;
  List<api.ContactGroup> contacts;
  List<api.ContactGroup> inactiveCon;
  List<api.LeadGroup> inactiveLead;

  List<dynamic> active;
  List<dynamic> inactive = [];

  @override
  void initState() {
    super.initState();

    if (widget.type == 'Lead') {
      leads = widget.leadGroups ?? [];
    } else {
      contacts = widget.contactGroups ?? [];
    }

    init();
  }

  void init() async {
    inactiveCon = [];
    inactiveLead = [];
    bool hit = false;
    if (widget.type == 'Lead') {
      final groups = await widget.userInfo.client.listLeadGroups(totals: true);
      for (var group in groups) {
        hit = false;
        for (var iGroup in leads) {
          if (group.name == iGroup.name) {
            hit = true;
          }
        }
        if (!hit) {
          inactiveLead.add(group);
        }
      }
    } else if (widget.type == 'Contact') {
      final groups =
          await widget.userInfo.client.listContactGroups(totals: true);
      for (var group in groups) {
        hit = false;
        for (var iGroup in contacts) {
          if (group.name == iGroup.name) {
            hit = true;
          }
        }
        if (!hit) {
          inactiveCon.add(group);
        }
      }
    }
    setState(() {});
  }

  void editGroups() async {
    if (widget.type == 'Lead') {
      if (widget.returnList) {
        Navigator.pop(context, leads);
      } else {
        await widget.userInfo.client.editLead(widget.lead.leadId,
            groups: leads,
            firstName: widget.lead.firstName,
            lastName: widget.lead.lastName);
      }
    } else if (widget.type == 'Contact') {
      if (widget.returnList) {
        Navigator.pop(context, contacts);
      } else {
        await widget.userInfo.client.editContact(widget.contact.contactId,
            groups: contacts,
            firstName: widget.contact.firstName,
            lastName: widget.contact.lastName);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('${widget.type} Groups'),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.check),
            onPressed: () {
              editGroups();
            },
          )
        ],
      ),
      body: Container(
        child: SafeArea(
          child: SingleChildScrollView(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Container(
                  padding: EdgeInsets.all(10),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text('Current Groups',
                          style: Theme.of(context).textTheme.subhead),
                      Builder(
                        builder: (_) {
                          if (widget.type == 'Lead') {
                            if (leads.isEmpty)
                              return Center(
                                child: Text('Add Group'),
                              );
                            return Wrap(
                              children: <Widget>[
                                for (var group in leads) ...[
                                  Padding(
                                    padding: const EdgeInsets.all(4.0),
                                    child: Chip(
                                      backgroundColor: Colors.lightGreen,
                                      deleteIcon: Icon(Icons.close),
                                      onDeleted: () {
                                        setState(() {
                                          leads.remove(group);
                                          inactiveLead.add(group);
                                          inactiveLead.sort((a, b) =>
                                              a.name.compareTo(b.name));
                                        });
                                      },
                                      label: Text(group.name),
                                    ),
                                  )
                                ],
                              ],
                            );
                          } else if (widget.type == 'Contact') {
                            if (contacts.isEmpty)
                              return Center(
                                child: Text('Add Group'),
                              );
                            return Wrap(
                              children: <Widget>[
                                for (var group in contacts) ...[
                                  Padding(
                                    padding: const EdgeInsets.all(4.0),
                                    child: Chip(
                                      backgroundColor: Colors.lightGreen,
                                      deleteIcon: Icon(Icons.close),
                                      onDeleted: () {
                                        setState(() {
                                          contacts.remove(group);
                                          inactiveCon.add(group);
                                          inactiveCon.sort((a, b) =>
                                              a.name.compareTo(b.name));
                                        });
                                      },
                                      label: Text(group.name),
                                    ),
                                  )
                                ],
                              ],
                            );
                          }
                        },
                      )
                    ],
                  ),
                ),
                Divider(),
                Container(
                    padding: EdgeInsets.all(10),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Text('Available Groups',
                            style: Theme.of(context).textTheme.subhead),
                        Builder(builder: (_) {
                          if (widget.type == 'Lead') {
                            return Wrap(
                              children: <Widget>[
                                for (var group in inactiveLead) ...[
                                  Padding(
                                    padding: EdgeInsets.all(4),
                                    child: GestureDetector(
                                      child: Chip(
                                        label: Text(group.name),
                                      ),
                                      onTap: () {
                                        setState(() {
                                          inactiveLead.remove(group);
                                          leads.add(group);
                                        });
                                      },
                                    ),
                                  ),
                                ]
                              ],
                            );
                          } else if (widget.type == 'Contact') {
                            return Wrap(
                              children: <Widget>[
                                for (var group in inactiveCon) ...[
                                  Padding(
                                    padding: EdgeInsets.all(4),
                                    child: GestureDetector(
                                      child: Chip(
                                        label: Text(group.name),
                                      ),
                                      onTap: () {
                                        setState(() {
                                          inactiveCon.remove(group);
                                          contacts.add(group);
                                        });
                                      },
                                    ),
                                  ),
                                ]
                              ],
                            );
                          }
                        })
                      ],
                    ))
              ],
            ),
          ),
        ),
      ),
    );
  }
}
