import 'package:flutter/material.dart';
import 'package:unify_api/tool.dart';

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/classes/address.dart';
import '../../../data/classes/phone.dart';
import '../../../data/utils/navigate.dart';
import '../../common/app_input_field.dart';
import '../../common/manage_groups.dart';
import '../../common/phone_tile.dart';
import 'address_tile.dart';

class AddEditContact extends StatefulWidget {
  AddEditContact({
    Key key,
    this.contact,
  }) : super(key: key);

  final ContactDetails contact;

  @override
  _AddEditContactState createState() => _AddEditContactState();
}

class _AddEditContactState extends State<AddEditContact> {
  Phone cellPhone, homePhone, officePhone;
  PhoneNumber cell, home, office;
  Address currentAddress;
  PostalAddress current;
  CompanyCategory category;
  List<ContactGroup> contactGroups;
  final TextEditingController nameController = TextEditingController();
  final TextEditingController lNameController = TextEditingController();
  final TextEditingController emailController = TextEditingController();
  final FocusNode nameFocus = FocusNode();
  final FocusNode lNameFocus = FocusNode();
  final FocusNode emailFocus = FocusNode();

  List<String> companyCategory = [
    'Realtor Partner',
    'Buisiness Contact',
    'New Recruit',
    'Personal Contact',
    'Previous Borrower',
    'Builder'
  ];
  String catSelected;
  bool isEdit = false;

  @override
  void initState() {
    isEdit = false;
    catSelected = companyCategory.first;
    cellPhone = null;
    homePhone = null;
    officePhone = null;
    currentAddress = null;
    cell = null;
    home = null;
    office = null;
    current = null;
    contactGroups = [];
    category = null;
    if (widget.contact != null) {
      isEdit = true;
      setFeilds();
    }
  }

  void setFeilds() {
    if (widget.contact.firstName != null)
      nameController.text = widget.contact.firstName;
    if (widget.contact.lastName != null)
      lNameController.text = widget.contact.lastName;
    if (widget.contact.email != null)
      emailController.text = widget.contact.email;
    if (widget.contact.companyCategory.name != null)
      catSelected = widget.contact.companyCategory.name;
    if (widget.contact.cellPhone.toString() != null) {
      cellPhone = Phone.fromJson(widget.contact.cellPhone.json);
    }
    if (widget.contact.homePhone.toString() != null) {
      homePhone = Phone.fromJson(widget.contact.homePhone.json);
    }
    if (widget.contact.officePhone.toString() != null) {
      officePhone = Phone.fromJson(widget.contact.officePhone.json);
    }
    if (widget.contact.currentAddress.toString() != null) {
      currentAddress = Address.fromJson(widget.contact.currentAddress.json);
    }
  }

  void getValues(UnifyClient client) {
    if (currentAddress != null) {
      current = PostalAddress(client, currentAddress.toJson());
    }
    if (cellPhone != null) {
      cell = PhoneNumber(client, cellPhone.toJson());
    }
    if (homePhone != null) {
      home = PhoneNumber(client, homePhone.toJson());
    }
    if (officePhone != null) {
      office = PhoneNumber(client, officePhone.toJson());
    }
    final catJson = <String, dynamic>{'name': catSelected};
    category = CompanyCategory(client, catJson);
  }

  @override
  Widget build(BuildContext context) {
    final userInfo = AuthBloc.currentInfo(context);
    return Scaffold(
      appBar: AppBar(
        title: isEdit ? Text('Edit Contact') : Text('New Contact'),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.check),
            onPressed: () async {
              if (isEdit) {
                getValues(userInfo.client);
                await userInfo.client.editContact(widget.contact.id,
                    firstName: nameController.value.text,
                    lastName: lNameController.value.text,
                    email: emailController.value.text,
                    currentAddress: current,
                    cellPhone: cell,
                    homePhone: home,
                    officePhone: office,
                    groups: contactGroups);
              } else {
                getValues(userInfo.client);
                await userInfo.client.addContact(
                    firstName: nameController.value.text,
                    lastName: lNameController.value.text,
                    email: emailController.value.text,
                    currentAddress: current,
                    cellPhone: cell,
                    homePhone: home,
                    officePhone: office,
                    groups: contactGroups,
                    companyCategory: category);
              }
            },
          )
        ],
      ),
      body: SingleChildScrollView(
        child: SafeArea(
          child: Container(
            child: Column(
              children: <Widget>[
                AppInputField(
                  controller: nameController,
                  name: 'First Name',
                  multiLine: false,
                  autoFocus: false,
                  focus: nameFocus,
                ),
                AppInputField(
                  controller: lNameController,
                  name: 'Last Name',
                  multiLine: false,
                  autoFocus: false,
                  focus: lNameFocus,
                ),
                AppInputField(
                  controller: emailController,
                  name: 'Email Address',
                  multiLine: false,
                  autoFocus: false,
                  focus: emailFocus,
                ),
                Padding(
                  padding: const EdgeInsets.only(top: 8.0),
                  child: ListTile(
                    leading: Icon(Icons.label),
                    title: Text('Select a Category'),
                    subtitle: DropdownButton<String>(
                      value: catSelected,
                      items: companyCategory
                          .map(
                            (String item) => DropdownMenuItem<String>(
                                value: item,
                                child: SizedBox(
                                    child: Text(
                                  item,
                                  textScaleFactor: 1,
                                ))),
                          )
                          .toList(),
                      onChanged: (val) {
                        setState(() {
                          catSelected = val;
                        });
                      },
                    ),
                  ),
                ),
                if(!isEdit)
                  ListTile(
                      leading: Icon(Icons.people),
                      title: Text('Manage Contact Groups'),
                      subtitle: Text('Tap to Edit'),
                      onTap: () {
                        navigate<List<ContactGroup>>(
                            context,
                            ManageGroups(
                              returnList: true,
                              type: 'Contact',
                              userInfo: userInfo,
                              contactGroups: contactGroups,
                            )).then((list) {
                          contactGroups = list;
                        });
                      }),
                ExpansionTile(
                  initiallyExpanded: true,
                  title: Text('Phone Numbers'),
                  children: <Widget>[
                    PhoneInputTile(
                      label: 'Cell Phone',
                      number: cellPhone,
                      numberChanged: (val) {
                        setState(() {
                          cellPhone = val;
                        });
                      },
                    ),
                    PhoneInputTile(
                      label: 'Home Phone',
                      numberChanged: (val) {
                        setState(() {
                          homePhone = val;
                        });
                      },
                    ),
                    PhoneInputTile(
                      label: 'Office Phone',
                      showExt: true,
                      numberChanged: (val) {
                        setState(() {
                          officePhone = val;
                        });
                      },
                    )
                  ],
                ),
                ExpansionTile(
                  initiallyExpanded: true,
                  title: Text('Address Info'),
                  children: <Widget>[
                    AddressInputTile(
                      label: 'Current Address',
                      addressChanged: (Address value) {
                        setState(() {
                          currentAddress = value;
                        });
                      },
                      address: currentAddress,
                    )
                  ],
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
