import 'package:flutter/material.dart';

import '../../../data/services/database/database.dart';
import '../../../data/utils/date_formatter.dart';
import 'more_button.dart';

class DetailedTile extends StatelessWidget {
  DetailedTile({
    Key key,
    @required Contact contact,
    this.name,
    this.subtitle,
  })  : _contact = contact,
        super(key: key);

  final String name;
  final String subtitle;
  final Contact _contact;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.all(8),
      child: LayoutBuilder(
        builder: (context, dimens) => SizedBox(
          height: 100,
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Icon(Icons.account_circle, size: 45),
              Expanded(
                  child: Padding(
                      padding: EdgeInsets.only(left: 10, right: 10),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Text(
                            name,
                            style: Theme.of(context).textTheme.title,
                            maxLines: 1,
                          ),
                          Padding(padding: EdgeInsets.only(bottom: 6)),
                          Text(
                            subtitle,
                            style: Theme.of(context).textTheme.subtitle,
                            maxLines: 1,
                          ),
                          if (dimens.maxWidth < 450 && dimens.maxWidth > 340)
                            _buildMore(),
                        ],
                      ))),
              if (dimens.maxWidth >= 450) _buildMore(),
              Column(
                crossAxisAlignment: CrossAxisAlignment.end,
                children: <Widget>[
                  MoreButton(
                    cell: _contact.cellPhone,
                    email: _contact.email,
                    home: _contact.homePhone,
                    office: _contact.officePhone,
                  ),
                ],
              )
            ],
          ),
        ),
      ),
    );
  }

  Row _buildMore() {
    return Row(
      children: <Widget>[
        TileAction(
          label: formatDateCustom(_contact.dateCreated, format: "MM-dd-yy"),
          tooltip: "Date Created",
          avatar: Icon(Icons.event),
        ),
        TileAction(
          label: formatDateCustom(_contact.lastUpdated, format: "MM-dd-yy"),
          tooltip: "Date Modified",
          avatar: Icon(Icons.event),
        ),
      ],
    );
  }
}

class TileAction extends StatelessWidget {
  TileAction({
    this.avatar,
    @required this.label,
    this.tooltip,
    this.onPressed,
  });
  final String label;
  final Icon avatar;
  final String tooltip;
  final VoidCallback onPressed;
  @override
  Widget build(BuildContext context) {
    if (label == null || label.isEmpty) return Container();
    if (onPressed == null) {
      return Container(
        padding: EdgeInsets.symmetric(horizontal: 1.0),
        child: Tooltip(
          message: tooltip ?? "",
          child: Chip(
            avatar: avatar,
            label: Text(label),
          ),
        ),
      );
    }
    return Container(
      padding: EdgeInsets.symmetric(horizontal: 1.0),
      child: ActionChip(
        elevation: 4,
        tooltip: tooltip ?? "",
        avatar: avatar,
        label: Text(label),
        onPressed: () {},
      ),
    );
  }
}
