import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import 'index.dart';

class ContactFavoritesScreen extends StatelessWidget {

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    return Scaffold(
      body: SafeArea(
        child: Container(
          child: StreamBuilder<List<Favorite>>(
              stream: GetIt.instance<DatabaseService>()
                  .favoritesDao
                  .watchFavoriteByType(user.userID, 'contact'),
              builder: (context, list) {
                if (!list.hasData || list.data == null) {
                  return Center(child: CircularProgressIndicator());
                } else {
                  if (list.data.isEmpty) {
                    return Center(child: Text('No Favorites'));
                  }
                  return SingleChildScrollView(
                    child: Column(
                      children: <Widget>[
                        for (var fav in list.data) ...[
                          FutureBuilder<Contact>(
                            future: GetIt.instance<DatabaseService>()
                                .contactsDao
                                .getContact(user.userID, fav.typeId),
                            builder: (context, contact) {
                              if (contact.data != null && contact.hasData) {
                                return Container(
                                  child: ContactRowTile(
                                    contact: contact.data,
                                    type: ViewType.dense,
                                    border: _buildBorder(Colors.black),
                                    iconColor: Colors.black,
                                    favorite: true,
                                    favId: fav.favoriteId,
                                  ),
                                );
                              }
                              return Container();
                            },
                          ),
                        ]
                      ],
                    ),
                  );
                }
              }),
        ),
      ),
    );
  }

  BoxDecoration _buildBorder(Color color) {
    return BoxDecoration(
        border: Border(
      left: BorderSide(
        color: color,
        width: 5.0,
      ),
    ));
  }
}
