import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';

import '../../common/index.dart';
import '../../common/tracker_list.dart';
import 'index.dart';

class TimelineList extends StatelessWidget {
  TimelineList({Key key, this.contactDetails, this.leadDetails})
      : super(key: key);

  final ContactDetails contactDetails;
  final LeadDetails leadDetails;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Timeline'),
      ),
      floatingActionButton: ContactsActionAdd(
        contactDetails: contactDetails,
        leadDetails: leadDetails,
      ),
      body: Container(
        child: SafeArea(
          child: SingleChildScrollView(
            child: FutureBuilder<List<Tracker>>(
                future: _getTracker(),
                builder: (context, snapshot) {
                  if (!snapshot.hasData || snapshot.data == null) {
                    print('failed');
                    return Center(child: CircularProgressIndicator());
                  }
                  return Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    //mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      if (snapshot.data.isEmpty) ...[
                        Center(child: Text('No Events Found')),
                      ] else ...[
                        for (var event in snapshot.data) ...[TrackerTile(event)]
                      ]
                    ],
                  );
                }),
          ),
        ),
      ),
    );
  }

  Future<List<Tracker>> _getTracker() {
    if (contactDetails != null) {
      return contactDetails.listTracker();
    }
    if (leadDetails != null) {
      return leadDetails.listTracker();
    }
    return null;
  }
}
