import 'package:flutter/material.dart';

class ResponsiveScaffold extends StatelessWidget {
  const ResponsiveScaffold({
    this.scaffoldKey,
    this.drawer,
    this.endDrawer,
    this.title,
    this.body,
    this.actions,
    this.floatingActionButton,
    this.menuIcon,
    this.endIcon,
    this.kTabletBreakpoint = 720.0,
    this.kDesktopBreakpoint = 1440.0,
    this.hideAppBar = false,
  });

  final List<Widget> actions;
  final Widget body;
  final IconData menuIcon, endIcon;
  final FloatingActionButton floatingActionButton;
  final bool hideAppBar;
  final double kDesktopBreakpoint;
  final double kTabletBreakpoint;
  final Key scaffoldKey;
  final Widget title;

  final _drawerWidth = 304.0;

  final Widget Function(BuildContext, bool) drawer, endDrawer;

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (_, constraints) {
        if (constraints.maxWidth >= kDesktopBreakpoint) {
          return Material(
            child: Stack(
              children: <Widget>[
                Row(
                  children: <Widget>[
                    if (drawer != null) ...[
                      SizedBox(
                        width: _drawerWidth,
                        child: drawer(context, true),
                      ),
                    ],
                    Expanded(
                      child: Scaffold(
                        key: scaffoldKey,
                        appBar: hideAppBar
                            ? null
                            : AppBar(
                                automaticallyImplyLeading: false,
                                title: title,
                                actions: <Widget>[
                                  if (actions != null) ...actions,
                                ],
                              ),
                        body: Row(
                          children: <Widget>[
                            Expanded(
                              child: body ?? Container(),
                            ),
                            if (endDrawer != null) ...[
                              Container(
                                width: _drawerWidth,
                                child: Drawer(
                                  elevation: 3.0,
                                  child: SafeArea(
                                    child: endDrawer(context, true),
                                  ),
                                ),
                              ),
                            ],
                          ],
                        ),
                      ),
                    ),
                  ],
                ),
                if (floatingActionButton != null) ...[
                  Positioned(
                    top: 100.0,
                    left: _drawerWidth - 30,
                    child: floatingActionButton,
                  )
                ],
              ],
            ),
          );
        }
        if (constraints.maxWidth >= kTabletBreakpoint) {
          return Material(
            child: Stack(
              children: <Widget>[
                Row(
                  children: <Widget>[
                    if (drawer != null) ...[
                      SizedBox(
                        width: _drawerWidth,
                        child: drawer(context, true),
                      ),
                    ],
                    Expanded(
                      child: Scaffold(
                        key: scaffoldKey,
                        endDrawer: endDrawer == null
                            ? null
                            : Drawer(child: endDrawer(context, false)),
                        appBar: hideAppBar
                            ? null
                            : AppBar(
                                automaticallyImplyLeading: false,
                                title: title,
                                actions: <Widget>[
                                  if (actions != null) ...actions,
                                  if (endDrawer != null) ...[
                                    _OptionsButton(iconData: endIcon),
                                  ]
                                ],
                              ),
                        body: Row(
                          children: <Widget>[
                            Expanded(
                              child: body ?? Container(),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ],
                ),
                if (floatingActionButton != null) ...[
                  Positioned(
                    top: 100.0,
                    left: _drawerWidth - 30,
                    child: floatingActionButton,
                  )
                ],
              ],
            ),
          );
        }
        return Scaffold(
          key: scaffoldKey,
          drawer: drawer == null ? null : drawer(context, false),
          endDrawer: endDrawer == null
              ? null
              : Drawer(
                  child: endDrawer(context, false),
                ),
          appBar: hideAppBar
              ? null
              : AppBar(
                  automaticallyImplyLeading: false,
                  leading: _MenuButton(iconData: menuIcon),
                  title: title,
                  actions: <Widget>[
                    if (actions != null) ...actions,
                    if (endDrawer != null) ...[
                      _OptionsButton(iconData: endIcon),
                    ]
                  ],
                ),
          body: body,
          floatingActionButton: floatingActionButton,
          floatingActionButtonLocation: FloatingActionButtonLocation.endFloat,
        );
      },
    );
  }
}

class _OptionsButton extends StatelessWidget {
  const _OptionsButton({
    Key key,
    @required this.iconData,
  }) : super(key: key);

  final IconData iconData;

  @override
  Widget build(BuildContext context) {
    return IconButton(
      icon: Icon(iconData ?? Icons.more_vert),
      onPressed: () {
        Scaffold.of(context).openEndDrawer();
      },
    );
  }
}

class _MenuButton extends StatelessWidget {
  const _MenuButton({
    Key key,
    @required this.iconData,
  }) : super(key: key);

  final IconData iconData;

  @override
  Widget build(BuildContext context) {
    return IconButton(
      icon: Icon(iconData ?? Icons.menu),
      onPressed: () {
        Scaffold.of(context).openDrawer();
      },
    );
  }
}
