import 'package:flutter/material.dart';
import 'package:unify_api/tool.dart';

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/classes/address.dart';
import '../../../data/classes/phone.dart';
import '../../../data/utils/navigate.dart';
import '../../common/app_input_field.dart';
import '../../common/manage_groups.dart';
import '../../common/phone_tile.dart';
import '../../contacts/common/address_tile.dart';

class AddEditLead extends StatefulWidget {
  AddEditLead({
    Key key,
    this.lead,
  }) : super(key: key);

  final LeadDetails lead;

  @override
  _AddEditLeadState createState() => _AddEditLeadState();
}

class _AddEditLeadState extends State<AddEditLead> {
  Phone cellPhone, homePhone, officePhone;
  PhoneNumber cell, home, office;
  Address currentAddress, propAddress;
  PostalAddress current, property;
  List<LeadGroup> leadGroups;
  final TextEditingController nameController = TextEditingController();
  final TextEditingController lNameController = TextEditingController();
  final TextEditingController emailController = TextEditingController();
  final TextEditingController grossController = TextEditingController();
  final TextEditingController bestTimeController = TextEditingController();
  final FocusNode nameFocus = FocusNode();
  final FocusNode lNameFocus = FocusNode();
  final FocusNode emailFocus = FocusNode();
  final FocusNode grossFocus = FocusNode();
  final FocusNode bestTimeFocus = FocusNode();

  List<String> creditCategory = [
    'None Selected',
    '740-850 Excellent',
    '720-739 Very Good',
    '680-719 Good',
    '640-679 Fair',
    '630-639 Low',
  ];
  String creditSelected;
  bool isEdit = false, bankrupt = false;

  @override
  void initState() {
    isEdit = false;
    creditSelected = creditCategory.first;
    cellPhone = null;
    homePhone = null;
    officePhone = null;
    currentAddress = null;
    propAddress = null;
    current = null;
    property = null;
    cell = null;
    home = null;
    office = null;
    leadGroups = [];
    if (widget.lead != null) {
      isEdit = true;
      setFeilds();
    }
  }

  void setFeilds() {
    if (widget.lead.firstName != null)
      nameController.text = widget.lead.firstName;
    if (widget.lead.lastName != null)
      lNameController.text = widget.lead.lastName;
    if (widget.lead.email != null) 
      emailController.text = widget.lead.email;
    if (widget.lead.annualGrossIncome != null)
      grossController.text = widget.lead.annualGrossIncome.toString();
    if (widget.lead.recentBankruptcy != null)
      bankrupt = widget.lead.recentBankruptcy;
    if (widget.lead.cellPhone.toString() != null) {
      cellPhone = Phone.fromJson(widget.lead.cellPhone.json);
    }
    if (widget.lead.homePhone.toString() != null) {
      homePhone = Phone.fromJson(widget.lead.homePhone.json);
    }
    if (widget.lead.officePhone.toString() != null) {
      officePhone = Phone.fromJson(widget.lead.officePhone.json);
    }
    if (widget.lead.currentAddress.toString() != null) {
      currentAddress = Address.fromJson(widget.lead.currentAddress.json);
    }
    if (widget.lead.propertyAddress.toString() != null) {
      propAddress = Address.fromJson(widget.lead.propertyAddress.json);
    }
  }

  void getValues(UnifyClient client) {
    if (currentAddress != null) {
      current = PostalAddress(client, currentAddress.toJson());
    }
    if (propAddress != null) {
      property = PostalAddress(client, propAddress.toJson());
    }
    if (cellPhone != null) {
      cell = PhoneNumber(client, cellPhone.toJson());
    }
    if (homePhone != null) {
      home = PhoneNumber(client, homePhone.toJson());
    }
    if (officePhone != null) {
      office = PhoneNumber(client, officePhone.toJson());
    }
  }

  @override
  Widget build(BuildContext context) {
    final userInfo = AuthBloc.currentInfo(context);
    return Scaffold(
      appBar: AppBar(
        title: isEdit ? Text('Edit Lead') : Text('New Lead'),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.check),
            onPressed: () async {
              if (isEdit) {
                getValues(userInfo.client);
                await userInfo.client.editLead(
                  widget.lead.id,
                  firstName: nameController.value.text,
                  lastName: lNameController.value.text,
                  email: emailController.value.text,
                  creditRange: creditSelected,
                  bestCallTime: bestTimeController.value.text,
                  annualGrossIncome: double.tryParse(grossController.value.text),
                  recentBankruptcy: bankrupt,
                  currentAddress: current,
                  propertyAddress: property,
                  cellPhone: cell,
                  groups: leadGroups,
                  homePhone: home,
                  officePhone: office,
                );
              } else {
                getValues(userInfo.client);
                await userInfo.client.addLead(
                  firstName: nameController.value.text,
                  lastName: lNameController.value.text,
                  email: emailController.value.text,
                  creditRange: creditSelected,
                  bestCallTime: bestTimeController.value.text,
                  annualGrossIncome: double.tryParse(grossController.value.text),
                  recentBankruptcy: bankrupt,
                  currentAddress: current,
                  propertyAddress: property,
                  cellPhone: cell,
                  homePhone: home,
                  officePhone: office,
                  groups: leadGroups
                );
                print('Pass');
              }
            },
          )
        ],
      ),
      body: SingleChildScrollView(
        child: SafeArea(
          child: Container(
            child: Column(
              children: <Widget>[
                AppInputField(
                  controller: nameController,
                  name: 'First Name',
                  multiLine: false,
                  autoFocus: false,
                  focus: nameFocus,
                ),
                AppInputField(
                  controller: lNameController,
                  name: 'Last Name',
                  multiLine: false,
                  autoFocus: false,
                  focus: lNameFocus,
                ),
                AppInputField(
                  controller: emailController,
                  name: 'Email Address',
                  multiLine: false,
                  autoFocus: false,
                  focus: emailFocus,
                ),
                ExpansionTile(
                  initiallyExpanded: true,
                  title: Text('Phone Numbers'),
                  children: <Widget>[
                    PhoneInputTile(
                      label: 'Cell Phone',
                      number: cellPhone,
                      numberChanged: (val) {
                        setState(() {
                          cellPhone = val;
                        });
                      },
                    ),
                    PhoneInputTile(
                      label: 'Home Phone',
                      numberChanged: (val) {
                        setState(() {
                          homePhone = val;
                        });
                      },
                    ),
                    PhoneInputTile(
                      label: 'Office Phone',
                      showExt: true,
                      numberChanged: (val) {
                        setState(() {
                          officePhone = val;
                        });
                      },
                    )
                  ],
                ),
                ExpansionTile(
                  initiallyExpanded: true,
                  title: Text('Address Info'),
                  children: <Widget>[
                    AddressInputTile(
                      label: 'Current Address',
                      addressChanged: (Address value) {
                        setState(() {
                          currentAddress = value;
                        });
                      },
                      address: currentAddress,
                    ),
                    AddressInputTile(
                      label: 'Property Address',
                      addressChanged: (Address value) {
                        setState(() {
                          currentAddress = value;
                        });
                      },
                      address: propAddress,
                    )
                  ],
                ),
                if(!isEdit)
                  ListTile(
                    leading: Icon(Icons.people),
                    title: Text('Manage Lead Groups'),
                    subtitle: Text('Tap to Edit'),
                    onTap: () {
                      navigate<List<LeadGroup>>(context, ManageGroups(
                        returnList: true,
                        type: 'Lead',
                        userInfo: userInfo,
                        leadGroups: leadGroups,
                      )).then((list){
                        leadGroups = list;
                      });
                    },
                  ),
                Divider(),
                ListTile(
                  leading: Text('Recent Bankruptcy'),
                  trailing: Switch.adaptive(
                    value: bankrupt,
                    onChanged: (value) {
                      setState(() {
                        bankrupt = value;
                      });
                    },
                  ),
                ),
                _buildDropDown('Credit Range'),
                AppInputField(
                  controller: grossController,
                  name: 'Annual Gross Income',
                  multiLine: false,
                  autoFocus: false,
                  focus: grossFocus,
                ),
                AppInputField(
                  controller: bestTimeController,
                  name: 'Best Call Time',
                  multiLine: false,
                  autoFocus: false,
                  focus: bestTimeFocus,
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  Padding _buildDropDown(String title) {
    return Padding(
      padding: const EdgeInsets.only(top: 8.0),
      child: ListTile(
        leading: Icon(Icons.label),
        title: Text(title),
        subtitle: DropdownButton<String>(
          value: creditSelected,
          items: creditCategory
              .map(
                (String item) => DropdownMenuItem<String>(
                    value: item,
                    child: SizedBox(
                        child: Text(
                      item,
                      textScaleFactor: 1,
                    ))),
              )
              .toList(),
          onChanged: (val) {
            setState(() {
              creditSelected = val;
            });
          },
        ),
      ),
    );
  }
}
