import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import '../../contacts/common/more_button.dart';
import 'detailed_lead_tile.dart';
import 'detailed_screen.dart';

class LeadRowTile extends StatelessWidget {
  const LeadRowTile({
    Key key,
    @required Lead lead,
    @required ViewType type,
    this.border,
    this.iconColor,
    this.onTap,
    this.onLongPress,
    this.favorite = false,
    this.favId,
    this.inGroup = false,
    this.groupId,
    this.itemId,
  })  : _lead = lead,
        _type = type,
        super(key: key);

  final Lead _lead;
  final ViewType _type;
  final BoxDecoration border;
  final Color iconColor;
  final bool favorite, inGroup;
  final int favId;
  final VoidCallback onTap, onLongPress;
  final String groupId, itemId;

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final info = AuthBloc.currentInfo(context);
    final _client = info.client;
    final leads = BlocProvider.of<LeadsBloc>(context);
    final searchBloc = BlocProvider.of<SearchBloc>(context);
    switch (_type) {
      case ViewType.detailed:
        return Slidable(
          actionPane: SlidableDrawerActionPane(),
          actionExtentRatio: 0.25,
          actions: <Widget>[
            if (inGroup)
              IconSlideAction(
                caption: 'Remove',
                color: Colors.red,
                icon: Icons.remove_circle_outline,
                onTap: () async {
                  final leadGroups = await _client
                      .getLeadDetails(_lead.leadId)
                      .then((details) {
                    final grps = details.groups;
                    for (var group in grps) {
                    if (group.id == groupId) {
                      grps.remove(group);
                    }
                    return grps;
                  }
                  });
                  print('TEST');
                  await GetIt.instance<DatabaseService>()
                      .groupItemDao
                      .removeItem(_lead.leadId, user.userID, groupId);
                  await _client.editLead(_lead.leadId,
                      groups: leadGroups,
                      firstName: _lead.firstName,
                      lastName: _lead.lastName);
                  
                },
              ),
            if (!inGroup)
              IconSlideAction(
                caption: 'Delete',
                color: Colors.red,
                icon: Icons.delete,
                onTap: () {
                  GetIt.instance<DatabaseService>()
                      .leadsDao
                      .deleteLead(user.userID, _lead.leadId);
                  searchBloc.dispatch(TextChanged(text: '', user: info));
                  _client.deleteLead(_lead.leadId);
                },
              )
          ],
          secondaryActions: <Widget>[
            if (!favorite)
              IconSlideAction(
                caption: 'Favorite',
                color: Colors.green,
                icon: Icons.star,
                onTap: () {
                  GetIt.instance<DatabaseService>()
                      .favoritesDao
                      .addItem('lead', _lead.leadId, user.userID);
                },
              ),
            if (favorite)
              IconSlideAction(
                caption: 'Unfavorite',
                color: Colors.green,
                icon: Icons.star_border,
                onTap: () {
                  GetIt.instance<DatabaseService>()
                      .favoritesDao
                      .removeItem(favId, user.userID);
                },
              )
          ],
          child: GestureDetector(
            onTap: () {
              Navigator.push(
                  context,
                  MaterialPageRoute(
                      builder: (context) => LeadsDetailScreen(lead: _lead)));
            },
            child: DetailedLeadTile(
              lead: _lead,
              name: '${_lead.firstName} ${_lead.lastName}',
              subtitle: _lead.lastActivity,
            ),
          ),
        );
        break;
      case ViewType.dense:
        return GestureDetector(
          onTap: () {
            Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => LeadsDetailScreen(lead: _lead)));
          },
          child: Slidable(
            actionPane: SlidableDrawerActionPane(),
            actionExtentRatio: 0.25,
            child: Container(
              decoration: border,
              child: ListTile(
                onLongPress: () {},
                onTap: () {
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                          builder: (context) =>
                              LeadsDetailScreen(lead: _lead)));
                },
                title: Text('${_lead.firstName}, ${_lead.lastName}'),
                subtitle: Text(_lead.lastActivity),
                leading: Icon(
                  Icons.people,
                  size: 34,
                  color: iconColor,
                ),
                trailing: MoreButton(
                  email: _lead.email,
                  cell: _lead.cellPhone,
                  office: _lead.officePhone,
                  home: _lead.homePhone,
                ),
              ),
            ),
            secondaryActions: <Widget>[
              if (!favorite)
                IconSlideAction(
                  caption: 'Favorite',
                  color: Colors.green,
                  icon: Icons.star,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .favoritesDao
                        .addItem('lead', _lead.leadId, user.userID);
                  },
                ),
              if (favorite)
                IconSlideAction(
                  caption: 'UnFavorite',
                  color: Colors.green,
                  icon: Icons.star_border,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .favoritesDao
                        .removeItem(favId, user.userID);
                  },
                )
            ],
          ),
        );
        break;
    }
    return Container();
  }
}
