import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_rounded_progress_bar/flutter_rounded_progress_bar.dart';
import 'package:flutter_rounded_progress_bar/rounded_progress_bar_style.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart' as unify;

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/is_empty.dart';
import '../../../data/utils/navigate.dart';
import '../../common/contacts_list.dart';
import '../../common/index.dart';
import '../../dashboard/cards/action_button.dart';
import 'calculator.dart';
import 'loan_graphs.dart';

class LoanDetailsScreen extends StatelessWidget {
  LoanDetailsScreen({Key key, @required this.loan, this.loanDetailsPass})
      : super(key: key);

  Loan loan;
  String loanDetailsPass;

  DateFormat formatter = new DateFormat('MM-dd-yyyy');
  final NumberFormat money = NumberFormat.simpleCurrency();
  unify.LoanDetails loanDetails;

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final userInfo = AuthBloc.currentInfo(context);
    final loans = BlocProvider.of<LoansBloc>(context);

    final origContext = context;

    if (loanDetailsPass == null) {
      loans.dispatch(UpdateLoanDetails(
          user: userInfo, loanId: loan.loanId, userId: user.userID));
    }
    return Scaffold(
      appBar: AppBar(
        title: Text('Loan Details'),
      ),
      body: SingleChildScrollView(
        child: Padding(
          padding: EdgeInsets.all(12),
          child: Builder(builder: (_) {
            if (loanDetailsPass == null) {
              return StreamBuilder<List<Loan>>(
                stream: GetIt.instance<DatabaseService>()
                    .loansDao
                    .watchLoansById(user.userID, loan.loanId),
                builder: (context, updateLoan) {
                  if (updateLoan.hasData &&
                      updateLoan.data[0].loanDetails != null) {
                    final loanInfo = updateLoan.data[0];
                    loanDetails = unify.LoanDetails(
                        userInfo.client,
                        json.decode(loanInfo.loanDetails),
                        loan.loanId);
                    return _buildPortraitMobile(context);
                  } else {
                    return Container(
                        child: Center(child: CircularProgressIndicator()));
                  }
                },
              );
            }
            print('Before:');
            print(loanDetailsPass);
            print('After:');
            print(loanDetailsPass.replaceAll('1800-01-01T00:00:00', ''));
            loanDetails = unify.LoanDetails(
                userInfo.client,
                json.decode(
                    loanDetailsPass.replaceAll('1800-01-01T00:00:00', '')),
                null);
            return _buildPortraitMobile(origContext);
          }),
        ),
      ),
    );
  }

  Column _buildPortraitMobile(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        _buildBarrowerInfo(loanDetails),
        _buildLoanInfo(loanDetails),
        _buildActionTiles(context, loanDetails),
        _buildRateLockCard(context, loanDetails),
        _buildProgressCard(context, loanDetails),
        _buildPieChart(context, loanDetails),
      ],
    );
  }

  RaisedChild _buildPieChart(
      BuildContext context, unify.LoanDetails loanDetails) {
    return RaisedChild(
      Container(
        child: Column(
          children: <Widget>[
            Padding(
                padding: EdgeInsets.all(10),
                child: Row(
                  children: <Widget>[
                    Icon(Icons.credit_card),
                    Text(
                      '  Monthly Payment',
                      style: Theme.of(context)
                          .textTheme
                          .title
                          .copyWith(fontWeight: FontWeight.w700),
                    )
                  ],
                )),
            Padding(
                padding: EdgeInsets.only(left: 24, right: 24, bottom: 24),
                child: Column(
                  children: <Widget>[
                    LoanPieChart(
                      details: loanDetails,
                    )
                  ],
                ))
          ],
        ),
      ),
    );
  }

  Padding _buildProgressCard(
      BuildContext context, unify.LoanDetails loanDetails) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10.0),
      child: RaisedChild(Container(
        child: Column(
          children: <Widget>[
            Padding(
              padding: EdgeInsets.all(10),
              child: Row(
                children: <Widget>[
                  Icon(Icons.timeline),
                  Text(
                    '  Loan Progress',
                    style: Theme.of(context)
                        .textTheme
                        .title
                        .copyWith(fontWeight: FontWeight.w700),
                  )
                ],
              ),
            ),
            Padding(
                padding: EdgeInsets.only(left: 24, right: 24, bottom: 24),
                child: Center(
                    child: RoundedProgressBar(
                        style: RoundedProgressBarStyle(
                          colorProgress: Theme.of(context).accentColor,
                          borderWidth: 0,
                          widthShadow: 0,
                        ),
                        borderRadius: BorderRadius.circular(24),
                        percent: (loanDetails.progress.value ?? 0) * 20.0,
                        childRight: Text(
                            loanDetails.progress.name ?? 'No Progress',
                            style: TextStyle(
                                color: Colors.white,
                                fontWeight: FontWeight.w600,
                                fontSize: 17)))))
          ],
        ),
      )),
    );
  }

  Padding _buildRateLockCard(
      BuildContext context, unify.LoanDetails loanDetails) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10.0),
      child: RaisedChild(
        Container(
          child: Column(
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: Row(
                  children: <Widget>[
                    Icon(Icons.lock),
                    Text(
                      '  Rate Lock',
                      style: Theme.of(context)
                          .textTheme
                          .title
                          .copyWith(fontWeight: FontWeight.w700),
                    )
                  ],
                ),
              ),
              Padding(
                padding: const EdgeInsets.only(left: 24, right: 24, bottom: 24),
                child: Column(
                  children: <Widget>[
                    ListTile(
                        leading: Icon(Icons.info),
                        title: Text('Rate Status'),
                        subtitle: Text(loanDetails.lockStatus ?? ' ')),
                    ListTile(
                        leading: Icon(Icons.calendar_today),
                        title: Text('Lock Date'),
                        subtitle: Text(formatter.format(loanDetails.lockDate))),
                    ListTile(
                        leading: Icon(Icons.access_time),
                        title: Text('Lock Expiration'),
                        subtitle:
                            Text(formatter.format(loanDetails.lockExpiration)))
                  ],
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  Padding _buildActionTiles(
      BuildContext context, unify.LoanDetails loanDetails) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          Container(
            width: MediaQuery.of(context).size.width / 2 - 18,
            child: ActionButtonCard(
                preview: true,
                title: 'Loan Contacts',
                subtitle: 'Total: ${loanDetails.contacts.length ?? '0'}',
                iconData: Icons.contacts,
                color: Colors.grey,
                onTap: () => navigate(
                    context, ContactsListView(contacts: loanDetails.contacts),
                    fullScreen: true)),
          ),
          Container(
            width: MediaQuery.of(context).size.width / 2 - 18,
            child: ActionButtonCard(
                preview: true,
                title: 'Calculator',
                subtitle: 'Mortgage Calculator',
                iconData: FontAwesomeIcons.calculator,
                fontawesome: true,
                color: Colors.black,
                onTap: () =>
                    navigate(context, CalculatorScreen(), fullScreen: true)),
          ),
        ],
      ),
    );
  }

  Padding _buildLoanInfo(unify.LoanDetails loanDetails) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10),
      child: RaisedChild(
        Container(
          padding: EdgeInsets.all(24),
          child: Column(
            children: <Widget>[
              ListTile(
                leading: Icon(Icons.folder),
                title: Text('Loan Number'),
                subtitle:
                    Text('${loanDetails.loanNumber ?? 'No Number Found'}'),
              ),
              ListTile(
                leading: Icon(Icons.attach_money),
                title: Text('Loan Amount'),
                subtitle: Text(
                    '${(loanDetails.loanAmount != null) ? money.format(loanDetails.loanAmount) : '\$0'}'),
              ),
              ListTile(
                leading: Icon(Icons.show_chart),
                title: Text('Interest Rate'),
                subtitle: Text('${loanDetails.rate ?? '0'} %'),
              ),
              // ListTile(
              //   leading: Icon(Icons.calendar_today),
              //   title: Text('Date Closed'),
              //   subtitle: Text(
              //       '${formatter.format(loanDetails.dateClosed) ?? 'No Date Found'}'),
              // )
            ],
          ),
        ),
      ),
    );
  }

  Padding _buildBarrowerInfo(unify.LoanDetails loanDetails) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10.0),
      child: RaisedChild(
        Container(
          padding: EdgeInsets.all(24),
          child: Column(
            children: <Widget>[
              ListTile(
                leading: Icon(Icons.person),
                title: Text('Barrower'),
                subtitle: Text(
                    '${(isNullOrEmpty(loanDetails.firstName)) ? 'No Barrower Found' : (loanDetails.firstName + ' ' + loanDetails.lastName)}'),
              ),
              ListTile(
                leading: Icon(Icons.person),
                title: Text('Co-Barrower'),
                subtitle: Text(
                    '${(isNullOrEmpty(loanDetails.coFristName)) ? 'No Co-Barrower Found' : (loanDetails.coFristName + ' ' + loanDetails.coLastName)}'),
              ),
              ListTile(
                leading: Icon(Icons.map),
                title: Text('Current Address'),
                subtitle: Text(formatAddress(current: true, loan: loanDetails)),
              ),
              ListTile(
                leading: Icon(Icons.map),
                title: Text('Property Address'),
                subtitle: Text(formatAddress(loan: loanDetails)),
              )
            ],
          ),
        ),
      ),
    );
  }

  String formatAddress({bool current = false, unify.LoanDetails loan}) {
    final address = current ? loan.currentAddress : loan.propertyAddress;
    String str;

    if (address == null) {
      return 'No Address Found';
    }
    try {
      str =
          '${address.street} ${address.city}\n${address.state} ${address.zip}';
    } catch (e) {
      str = 'No Address Found';
    }
    return str;
  }
}
