import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart';

import '../../../data/classes/calculator.dart';

class LoanPieChart extends StatelessWidget {
  LoanPieChart({this.details});

  final LoanDetails details;

  MortgageCalculation calculation;
  double total;
  final money = NumberFormat.simpleCurrency();
  List<PieChartSectionData> pieChartData = [];
  var section1 = PieChartSectionData(),
      section2 = PieChartSectionData(),
      section3 = PieChartSectionData(),
      section4 = PieChartSectionData();

  @override
  Widget build(BuildContext context) {
    return Container(
      child: FutureBuilder(
          future: calculateVal(),
          builder: (context, snapshot) {
            if (snapshot.hasData && snapshot.data == true) {
              return Column(
                children: <Widget>[
                  Container(
                    width: 200,
                    height: 170,
                    child: FlChart(
                      chart: PieChart(
                        PieChartData(
                            borderData: FlBorderData(show: false),
                            sectionsSpace: 0,
                            centerSpaceRadius: 0,
                            sections: pieChartData),
                      ),
                    ),
                  ),
                  Text(
                      'Total: ${money.format(total)}'),
                  _buildLegend()
                ],
              );
            }
            return Container(
              width: 200,
              height: 170,
              child: Center(
                child: Text('Not Enough Data'),
              ),
            );
          }),
    );
  }

  Container _buildLegend() {
    return Container(
        margin: const EdgeInsets.only(top: 20),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: <Widget>[
            buildListTile('Payment', calculation.monthlyPayment, Colors.red),
            if (pieChartData[1].value > 0)
              buildListTile('PMI', calculation.pmi, Colors.green),
            if (pieChartData[2].value > 0)
              buildListTile(
                  'Property Tax', calculation.propertyTaxes, Colors.blue),
            if (pieChartData[3].value > 0)
              buildListTile('HOA', calculation.hoaDues, Colors.yellow)
          ],
        ));
  }

  void buildGraph() {
    section1 = PieChartSectionData(
      color: Colors.red,
      value: calculation.monthlyPayment,
      title: '1',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      radius: 70,
    );
    section2 = PieChartSectionData(
      color: Colors.green,
      title: '2',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      value: calculation.pmi,
      radius: 70,
    );
    section3 = PieChartSectionData(
      color: Colors.blue,
      title: '3',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      value: calculation.propertyTaxes,
      radius: 70,
    );
    section4 = PieChartSectionData(
      color: Colors.yellow,
      title: '4',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      value: calculation.hoaDues,
      radius: 70,
    );

    List<PieChartSectionData> chartItems = [
      section1,
      section2,
      section3,
      section4,
    ];
    pieChartData = chartItems;
  }

  InkWell buildListTile(String title, double value, Color color,
      {VoidCallback onTap}) {
    return InkWell(
      onTap: onTap,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Padding(
            padding: const EdgeInsets.all(1.0),
            child: Icon(FontAwesomeIcons.solidCircle, size: 20, color: color),
          ),
          Padding(
            padding: const EdgeInsets.all(10.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Text(
                  title,
                  style: TextStyle(fontSize: 16),
                ),
                Text(
                  '${money.format(value)}',
                )
              ],
            ),
          ),
        ],
      ),
    );
  }

  Future<void> calculateVal() async {
    if (details.term == 0) {
      return false;
    }

    final purchasePrice = details.purchasePrice > 0
        ? details.purchasePrice
        : details.appraisedValue;

    final downPay =
        (purchasePrice - details.loanAmount) / details.loanAmount * 100;

    calculation = MortgageCalculation(
        loanPurpose: details.loanPurpose,
        loanTerm: details.term,
        downPayment: downPay,
        interestRate: details.rate,
        purchasePrice: purchasePrice,
        loanAmount: details.loanAmount,
        propertyTaxes: details.taxes,
        hoaDues: details.hoa);

    calculation.calculate();

    total = calculation.monthlyPayment +
        calculation.hoaDues +
        calculation.pmi +
        calculation.propertyTaxes;

    buildGraph();

    return true;
  }
}
