import 'package:example_db/data/classes/enum_list.dart';
import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import 'tile.dart';

class LoansSearchDelegate extends SearchDelegate<Loan> {
  LoansSearchDelegate(this.folder);
  final api.LoansFolder folder;

  @override
  ThemeData appBarTheme(BuildContext context) {
    return ThemeData.dark();
  }

  @override
  List<Widget> buildActions(BuildContext context) {
    return [
      IconButton(
        icon: Icon(Icons.clear),
        onPressed: () => query = '',
      ),
    ];
  }

  @override
  Widget buildLeading(BuildContext context) {
    return IconButton(
      icon: Icon(Icons.arrow_back),
      onPressed: () => close(context, null),
    );
  }

  @override
  Widget buildResults(BuildContext context) {
    final _user = AuthBloc.currentInfo(context);
    return _Results(
      user: _user,
      query: query,
      folder: folder,
      onSelected: (val) => close(context, val),
    );
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    final _user = AuthBloc.currentInfo(context);
    return _Results(
      user: _user,
      query: query,
      folder: folder,
      onSelected: (val) => close(context, val),
    );
  }
}

class _Results extends StatelessWidget {
  const _Results({
    Key key,
    @required api.AccountInfo user,
    @required this.query,
    @required this.folder,
    @required this.onSelected,
  })  : _user = user,
        super(key: key);

  final String query;
  final api.LoansFolder folder;
  final api.AccountInfo _user;
  final ValueChanged<Loan> onSelected;

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<List<Loan>>(
        future: GetIt.instance<DatabaseService>()
            .loansDao
            .searchLoans(_user.id, query, folder: folder),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            final _data = snapshot.data;
            if (_data.isEmpty) {
              return Center(child: Text('No Loans Found'));
            }
            return ListView.builder(
              itemCount: _data.length,
              itemBuilder: (context, index) {
                final _item = _data[index];
                return LoanRowTile(
                  loan: _item,
                  type: ViewType.detailed,
                  onTap: () => onSelected(_item),
                );
              },
            );
          }
          return Center(child: CircularProgressIndicator());
        });
  }
}
