import 'package:flutter/material.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import '../../contacts/common/more_button.dart';
import 'detailed_loan_tile.dart';
import 'detailed_screen.dart';

class LoanRowTile extends StatelessWidget {
  const LoanRowTile(
      {Key key,
      @required Loan loan,
      @required ViewType type,
      this.border,
      this.iconColor,
      this.onTap,
      this.onLongPress,
      this.favorite = false,
      this.favId})
      : _loan = loan,
        _type = type,
        super(key: key);

  final Loan _loan;
  final int favId;
  final bool favorite;
  final ViewType _type;
  final BoxDecoration border;
  final Color iconColor;
  final VoidCallback onTap, onLongPress;

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    switch (_type) {
      case ViewType.detailed:
        return GestureDetector(
          onTap: () => Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => LoanDetailsScreen(loan: _loan))),
          child: Slidable(
            actionPane: SlidableDrawerActionPane(),
            actionExtentRatio: 0.25,
            child: DetailedLoanTile(
              loan: _loan,
              name: '${_loan.firstName} ${_loan.lastName}',
              subtitle: _loan.loanMilestone,
            ),
            secondaryActions: <Widget>[
              if (!favorite)
                IconSlideAction(
                  caption: 'Favorite',
                  color: Colors.green,
                  icon: Icons.star,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .favoritesDao
                        .addItem('loan', _loan.loanId, user.userID);
                  },
                ),
              if (favorite)
                IconSlideAction(
                  caption: 'Unfavorite',
                  color: Colors.green,
                  icon: Icons.star_border,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .favoritesDao
                        .removeItem(favId, user.userID);
                  },
                ),
            ],
          ),
        );
        break;
      case ViewType.dense:
        final name = '${_loan.firstName}, ${_loan.lastName}';
        return GestureDetector(
          onTap: () {},
          child: Slidable(
            actionPane: SlidableDrawerActionPane(),
            actionExtentRatio: 0.25,
            child: Container(
              decoration: border,
              child: ListTile(
                onLongPress: () {},
                onTap: () => Navigator.push(
                    context,
                    MaterialPageRoute(
                        builder: (context) => LoanDetailsScreen(loan: _loan))),
                title:
                    Text(name.trim().isEmpty ? 'No Name Found' : name.trim()),
                subtitle: Text(_loan.loanMilestone),
                leading: Icon(
                  Icons.folder,
                  size: 34,
                  color: iconColor,
                ),
                trailing: MoreButton(
                  email: _loan.email,
                  cell: _loan.cellPhone,
                  office: _loan.officePhone,
                  home: _loan.homePhone,
                ),
              ),
            ),
            secondaryActions: <Widget>[
              if (!favorite)
                IconSlideAction(
                  caption: 'Favorite',
                  color: Colors.green,
                  icon: Icons.star,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .favoritesDao
                        .addItem('loan', _loan.loanId, user.userID);
                  },
                ),
              if (favorite)
                IconSlideAction(
                  caption: 'Unfavorite',
                  color: Colors.green,
                  icon: Icons.star_border,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .favoritesDao
                        .removeItem(favId, user.userID);
                  },
                ),
            ],
          ),
        );
        break;
    }
    return Container();
  }
}
