part of global_search;

class _SearchBar extends StatefulWidget {
  const _SearchBar({Key key, this.scopes}) : super(key: key);

  final List<SearchResultType> scopes;

  @override
  State<_SearchBar> createState() => _SearchBarState();
}

class _SearchBarState extends State<_SearchBar> {
  SearchBloc _githubSearchBloc;
  LocalStor storage;
  String _query = '';
  final List<SearchResultType> _scope = [
    SearchResultType.contacts,
    SearchResultType.leads,
    SearchResultType.loans,
    SearchResultType.alerts,
    SearchResultType.video
  ];

  final _textController = TextEditingController();

  @override
  void dispose() {
    _textController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    storage = new LocalStor('search')..init();
    if (widget?.scopes != null) {
      _scope.clear();
      _scope.addAll(widget.scopes);
    }
    _githubSearchBloc = BlocProvider.of<SearchBloc>(context);
  }

  void _onClearTapped(api.AccountInfo user) {
    _query = '';
    _textController.text = '';
    _githubSearchBloc
        .dispatch(TextChanged(text: '', user: user, types: _scope));
  }

  void _refreshSearch(api.AccountInfo user) {
    _githubSearchBloc
        .dispatch(TextChanged(text: _query, user: user, types: _scope));
  }

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentInfo(context);
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Card(
          elevation: 4,
          child: ExpansionTile(
            trailing: Icon(Icons.filter_list),
            title: TextField(
              controller: _textController,
              autocorrect: false,
              onChanged: (text) {
                _query = text;
                storage.setSavedItem('query', _query);
                _githubSearchBloc.dispatch(
                  TextChanged(text: _query, user: _user, types: _scope),
                );
              },
              decoration: InputDecoration(
                prefixIcon: Icon(Icons.search),
                suffixIcon: GestureDetector(
                  child: Icon(Icons.clear),
                  onTap: () => _onClearTapped(_user),
                ),
                border: InputBorder.none,
                hintText: 'Enter a search term',
              ),
            ),
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: Row(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Expanded(
                      child: Wrap(
                        runSpacing: 5,
                        spacing: 5,
                        children: <Widget>[
                          for (var type in SearchResultType.values) ...[
                            ActionChip(
                              label: Text(describeEnum(type)),
                              backgroundColor: _scope.contains(type)
                                  ? Colors.blueAccent.withAlpha(175)
                                  : null,
                              onPressed: () {
                                if (mounted) {
                                  setState(() {
                                    if (_scope.contains(type)) {
                                      _scope.remove(type);
                                    } else {
                                      _scope.add(type);
                                    }
                                  });
                                }
                                _githubSearchBloc.dispatch(
                                  TextChanged(
                                    text: _query,
                                    user: _user,
                                    types: _scope,
                                  ),
                                );
                              },
                            ),
                          ],
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }
}
