import 'package:example_db/data/services/database/database.dart';
import 'package:example_db/ui/account/screen.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_material_color_picker/flutter_material_color_picker.dart';
import 'package:get_it/get_it.dart';

import '../../data/blocs/bloc.dart';
import '../../data/blocs/settings/bloc.dart';
import '../auth/screen.dart';
import 'builder.dart';

class SettingsScreen extends StatefulWidget {
  @override
  _SettingsScreenState createState() => _SettingsScreenState();
}

class _SettingsScreenState extends State<SettingsScreen> {
  int tempColor = 0;

  // String _buildTitle(SettingTypes type) {
  //   switch (type) {
  //     case SettingTypes.darkMode:
  //       return 'Dark Mode';
  //     case SettingTypes.autoLogin:
  //       return 'Auto Login';
  //     case SettingTypes.contactColor:
  //       return 'Contact Indicator Color';
  //     case SettingTypes.loanColor:
  //       return 'Loan Indicator Color';
  //     case SettingTypes.leadColor:
  //       return 'Lead Indicator Color';
  //     case SettingTypes.alertColor:
  //       return 'Alert Indicator Color';
  //     case SettingTypes.videoColor:
  //       return 'Video Indicator Color';
  //     default:
  //       return describeEnum(type);
  //   }
  // }

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentInfo(context);
    final auth = BlocProvider.of<AuthBloc>(context);
    final nav = BlocProvider.of<NavBloc>(context);
    final db = GetIt.instance<DatabaseService>();
    return Scaffold(
      // appBar: AppBar(
      //   // leading: MenuButton(),
      //   title: Text('Settings'),
      //   actions: <Widget>[
      //     IconButton(
      //       icon: Icon(Icons.account_circle),
      //       onPressed: () => nav.dispatch(NavPushEvent(LoginScreen.routeName,
      //           args: const <String, dynamic>{
      //             'add': true,
      //           })),
      //     ),
      //     IconButton(
      //       icon: Icon(Icons.exit_to_app),
      //       onPressed: () => auth.dispatch(LogoutEvent(_user)),
      //     ),
      //   ],
      // ),
      body: ListView(
        children: <Widget>[
          ExpansionTile(
            leading: Icon(Icons.account_circle),
            title: Text('Account Settings'),
            initiallyExpanded: true,
            children: <Widget>[
              ListTile(
                leading: CircleAvatar(
                  child: Icon(Icons.account_circle),
                ),
                title: Text(_user.displayName),
                subtitle: Text(_user.title),
                trailing: IconButton(
                  icon: Icon(Icons.exit_to_app),
                  tooltip: 'Logout',
                  onPressed: () => auth.dispatch(LogoutEvent(_user)),
                ),
              ),
              SettingBuilder(
                builder: (context, settings, bloc) => SwitchListTile.adaptive(
                  title: Text('Auto Login'),
                  value: settings.autoLogin,
                  onChanged: (val) =>
                      bloc.dispatch(ChangeSettings(settings..autoLogin = val)),
                ),
              ),
            ],
          ),
          ExpansionTile(
            leading: Icon(Icons.people),
            title: Text('Manage Accounts'),
            initiallyExpanded: false,
            children: <Widget>[
              BlocBuilder<AuthBloc, AuthState>(
                builder: (context, state) {
                  return StreamBuilder<List<User>>(
                      stream: db.watchOtherUsers(_user.id),
                      builder: (context, snapshot) {
                        if (snapshot.hasData) {
                          return ListView.builder(
                            shrinkWrap: true,
                            itemCount: snapshot.data.length,
                            itemBuilder: (context, index) {
                              final _item = snapshot.data[index];
                              return ListTile(
                                leading: Icon(Icons.person),
                                title: Text(
                                    '${_item.firstName} ${_item.lastName}'),
                                subtitle: Text(_item.title),
                                onTap: () {
                                  BlocProvider.of<AuthBloc>(context)
                                      .dispatch(SwitchUser(_item));
                                },
                                onLongPress: () {
                                  Navigator.push(
                                      context,
                                      MaterialPageRoute(
                                          builder: (_) =>
                                              AccountScreen(user: _item)));
                                },
                                trailing: IconButton(
                                  icon: Icon(Icons.delete),
                                  onPressed: () => db.deleteUser(_item),
                                ),
                              );
                            },
                          );
                        }
                        return Container();
                      });
                },
              ),
              ListTile(
                title: Text('Add Account'),
                trailing: IconButton(
                  icon: Icon(Icons.add),
                  onPressed: () =>
                      Navigator.pushNamed(context, LoginScreen.addRoute),
                ),
              ),
            ],
          ),
          ExpansionTile(
            leading: Icon(Icons.brightness_medium),
            initiallyExpanded: true,
            title: Text('Theme Settings'),
            children: <Widget>[
              SettingBuilder(
                builder: (context, settings, bloc) => SwitchListTile.adaptive(
                  title: Text('Dark Mode'),
                  value: settings.darkMode,
                  onChanged: (val) =>
                      bloc.dispatch(ChangeSettings(settings..darkMode = val)),
                ),
              ),
              SettingBuilder(
                builder: (context, settings, bloc) => SwitchListTile.adaptive(
                  title: Text('System Setting'),
                  value: settings.useSystemDark,
                  onChanged: (val) => bloc
                      .dispatch(ChangeSettings(settings..useSystemDark = val)),
                ),
              ),
            ],
          ),
          ExpansionTile(
            leading: Icon(Icons.palette),
            title: Text('Color Options',
                style: Theme.of(context).textTheme.subhead),
            children: <Widget>[
              SettingBuilder(
                builder: (context, settings, bloc) => Padding(
                  padding: EdgeInsets.all(17),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                          child: Text(
                        'Contact Indicator Color',
                        style: Theme.of(context).textTheme.subhead,
                      )),
                      CircleAvatar(
                        backgroundColor: Color(settings.contactColor),
                        child: InkWell(
                          onTap: () {
                            _openColorPicker(
                              settings.contactColor,
                              (val) => bloc.dispatch(
                                  ChangeSettings(settings..contactColor = val)),
                            );
                          },
                        ),
                      )
                    ],
                  ),
                ),
              ),
              SettingBuilder(
                builder: (context, settings, bloc) => Padding(
                  padding: EdgeInsets.all(17),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                          child: Text(
                        'Loan Indicator Color',
                        style: Theme.of(context).textTheme.subhead,
                      )),
                      CircleAvatar(
                        backgroundColor: Color(settings.loanColor),
                        child: InkWell(
                          onTap: () {
                            _openColorPicker(
                              settings.loanColor,
                              (val) => bloc.dispatch(
                                  ChangeSettings(settings..loanColor = val)),
                            );
                          },
                        ),
                      )
                    ],
                  ),
                ),
              ),
              SettingBuilder(
                builder: (context, settings, bloc) => Padding(
                  padding: EdgeInsets.all(17),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                          child: Text(
                        'Lead Indicator Color',
                        style: Theme.of(context).textTheme.subhead,
                      )),
                      CircleAvatar(
                        backgroundColor: Color(settings.leadColor),
                        child: InkWell(
                          onTap: () {
                            _openColorPicker(
                              settings.leadColor,
                              (val) => bloc.dispatch(
                                  ChangeSettings(settings..leadColor = val)),
                            );
                          },
                        ),
                      )
                    ],
                  ),
                ),
              ),
              SettingBuilder(
                builder: (context, settings, bloc) => Padding(
                  padding: EdgeInsets.all(17),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                          child: Text(
                        'Alert Indicator Color',
                        style: Theme.of(context).textTheme.subhead,
                      )),
                      CircleAvatar(
                        backgroundColor: Color(settings.alertColor),
                        child: InkWell(
                          onTap: () {
                            _openColorPicker(
                              settings.alertColor,
                              (val) => bloc.dispatch(
                                  ChangeSettings(settings..alertColor = val)),
                            );
                          },
                        ),
                      )
                    ],
                  ),
                ),
              ),
              SettingBuilder(
                builder: (context, settings, bloc) => Padding(
                  padding: EdgeInsets.all(17),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                          child: Text(
                        'Video Indicator Color',
                        style: Theme.of(context).textTheme.subhead,
                      )),
                      CircleAvatar(
                        backgroundColor: Color(settings.videoColor),
                        child: InkWell(
                          onTap: () {
                            _openColorPicker(
                              settings.videoColor,
                              (val) => bloc.dispatch(
                                  ChangeSettings(settings..videoColor = val)),
                            );
                          },
                        ),
                      )
                    ],
                  ),
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }

  Future<T> _openDialog<T>(Widget content) {
    return showDialog<T>(
        context: context,
        builder: (_) {
          return AlertDialog(
            contentPadding: EdgeInsets.all(6),
            title: Text('Select a Color'),
            content: content,
          );
        });
  }

  void _openColorPicker(int color, ValueChanged<int> onChanged) async {
    await _openDialog(
      MaterialColorPicker(
          colors: fullMaterialColors,
          allowShades: false,
          selectedColor: Color(color),
          shrinkWrap: true,
          onMainColorChange: (color) {
            if (mounted)
              setState(() {
                tempColor = color.value;
                onChanged(tempColor);
              });
          }),
    );
  }
}
