/**
 * Creating custom color palettes is part of creating a custom app. The idea is to create
 * your class of custom colors, in this case `CompanyColors` and then create a `ThemeData`
 * object with those colors you just defined.
 *
 * Resource:
 * A good resource would be this website: http://mcg.mbitson.com/
 * You simply need to put in the colour you wish to use, and it will generate all shades
 * for you. Your primary colour will be the `500` value.
 *
 * Colour Creation:
 * In order to create the custom colours you need to create a `Map<int, Color>` object
 * which will have all the shade values. `const Color(0xFF...)` will be how you create
 * the colours. The six character hex code is what follows. If you wanted the colour
 * #114488 or #D39090 as primary colours in your theme, then you would have
 * `const Color(0x114488)` and `const Color(0xD39090)`, respectively.
 *
 * Usage:
 * In order to use this newly created theme or even the colours in it, you would just
 * `import` this file in your project, anywhere you needed it.
 * `import 'path/to/theme.dart';`
 */

import 'package:flutter/material.dart';

class AppColors {
  AppColors._(); // this basically makes it so you can instantiate this class

  // static const Map<int, Color> green = const <int, Color>{
  //   50: const Color(0xFFf2f8ef),
  //   100: const Color(0xFFdfedd8),
  //   200: const Color(0xFFc9e2be),
  //   300: const Color(0xFFb3d6a4),
  //   400: const Color(0xFFa3cd91),
  //   500: const Color(0xFF93c47d),
  //   600: const Color(0xFF8bbe75),
  //   700: const Color(0xFF80b66a),
  //   800: const Color(0xFF76af60),
  //   900: const Color(0xFF64a24d)
  // };

  static ThemeData get dark => ThemeData(
        fontFamily: 'ProductSans',
        brightness: Brightness.dark,
        primaryColor: Colors.black,
        accentColor: ThemeData.dark().accentColor,
        appBarTheme: ThemeData.dark().appBarTheme,
        bottomAppBarTheme:
            BottomAppBarTheme(color: Colors.black, elevation: 0.0),
        backgroundColor: Colors.black,
        scaffoldBackgroundColor: Colors.black,
      );

  static ThemeData get light => ThemeData(
        fontFamily: 'ProductSans',
        brightness: Brightness.light,
        // appBarTheme: AppBarTheme(
        //   brightness: Brightness.dark,
        //   color: Colors.white,
        //   iconTheme: IconThemeData(
        //     color: Colors.black,
        //   ),
        //   textTheme: TextTheme(
        //     title: TextStyle(
        //       color: Colors.black,
        //       fontWeight: FontWeight.w700,
        //       fontSize: 22,
        //     ),
        //   ),
        // ),
        cardTheme: CardTheme(
          elevation: 8.0,
        ),
        bottomAppBarTheme: BottomAppBarTheme(elevation: 0.0),
        tabBarTheme: TabBarTheme(
          labelColor: Colors.black,
        ),
        primaryColor: unifyBlue,
        accentColor: Colors.redAccent,
      );
  static Color get unifyBlue => Color.fromRGBO(80, 140, 196, 1);

  static bottomAppBarColor(BuildContext context) {
    if (Theme.of(context).brightness == Brightness.dark) {
      return Colors.black;
    }
    return Colors.white;
  }
}
