import 'package:example_db/data/utils/navigate.dart';
import 'package:example_db/ui/video/common/quick_send.dart';
import 'package:expandable/expandable.dart';
import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/intl.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../../plugins/video_player/video_player.dart';
import '../../common/app_input_field.dart';
import '../../common/raised_card.dart';

class DetailedVideoPage extends StatefulWidget {
  DetailedVideoPage(
      {Key key, @required Video video, this.title, this.description})
      : _video = video,
        super(key: key);

  final Video _video;
  final String title, description;

  bool edit = true;
  bool editExpanded = false;

  ExpandableController controller = ExpandableController();

  @override
  _DetailedVideoPageState createState() => _DetailedVideoPageState();
}

class _DetailedVideoPageState extends State<DetailedVideoPage> {
  final TextEditingController videoNameController = TextEditingController();
  final TextEditingController videoDescController = TextEditingController();
  final FocusNode nameFocus = FocusNode();
  final FocusNode descFocus = FocusNode();

  @override
  void initState() {
    super.initState();
    videoNameController.text = widget._video.name;
    videoDescController.text = widget._video.description;
  }

  @override
  Widget build(BuildContext context) {
    final userInfo = AuthBloc.currentInfo(context);
    final formatter = new DateFormat('LLL d, yyyy');
    //final _keyCard = GlobalKey<FlipCardState>();

    final client = userInfo.client;

    return Scaffold(
      appBar: AppBar(
        title: Text('Video Details'),
        actions: <Widget>[
          if (widget.edit && widget.controller.expanded)
            IconButton(
              icon: Icon(Icons.check),
              onPressed: () {
                client.editVideoInfo(widget._video.videoId,
                    name: videoNameController.value.text,
                    description: videoDescController.value.text);
                Navigator.pop(context);
              },
            )
        ],
      ),
      body: SafeArea(
        child: Container(
          padding: EdgeInsets.all(10),
          child: SingleChildScrollView(
            child: Column(
              children: <Widget>[
                Container(
                  child: Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: RaisedChild(
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          VideoPlayerView(
                            src: widget._video.download,
                            width: widget._video.width,
                            height: widget._video.height,
                          ),
                          Padding(
                            padding: const EdgeInsets.all(10.0),
                            child: ExpandablePanel(
                                controller: widget.controller,
                                hasIcon: false,
                                header: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: <Widget>[
                                    Container(
                                      width: 200,
                                      child: Text(widget._video.name,
                                          softWrap: true,
                                          maxLines: 2,
                                          overflow: TextOverflow.ellipsis,
                                          style: TextStyle(fontSize: 20)),
                                    ),
                                    Row(
                                      children: <Widget>[
                                        Text(
                                            '${widget._video.views.toString()} views'),
                                        Text(' | ',
                                            style: TextStyle(
                                                color: Colors.black
                                                    .withOpacity(0.3))),
                                        Text(formatter
                                            .format(widget._video.dateCreated))
                                      ],
                                    ),
                                  ],
                                ),
                                collapsed: Column(
                                  mainAxisAlignment: MainAxisAlignment.start,
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: <Widget>[
                                    Divider(),
                                    Text(
                                      widget._video.description,
                                      softWrap: true,
                                      maxLines: 2,
                                      overflow: TextOverflow.ellipsis,
                                    ),
                                  ],
                                ),
                                expanded: _buildMore(
                                    formatter,
                                    videoNameController,
                                    videoDescController,
                                    nameFocus,
                                    descFocus,
                                    edit: widget.edit)),
                          ),
                          Container(
                            padding: EdgeInsets.only(left: 10, right: 10),
                            //height: 20,
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: <Widget>[
                                Builder(builder: (_) {
                                  return GestureDetector(
                                    child: Text(
                                        widget.controller.expanded
                                            ? 'Less Info'
                                            : 'More Info',
                                        style: Theme.of(context)
                                            .textTheme
                                            .button
                                            .copyWith(fontSize: 16)),
                                    onTap: () {
                                      setState(() {
                                        widget.edit = false;
                                        widget.editExpanded = false;
                                      });
                                      widget.controller.toggle();
                                    },
                                  );
                                }),
                                GestureDetector(
                                    child: Padding(
                                      padding: const EdgeInsets.only(bottom: 8),
                                      child: Icon(
                                        Icons.settings,
                                        size: 20,
                                      ),
                                    ),
                                    onTap: () {
                                      setState(() {
                                        widget.edit = true;
                                        widget.editExpanded = true;
                                        widget.controller.toggle();
                                      });
                                    }),
                              ],
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
                ),
                Container(
                  child: Padding(
                    padding: EdgeInsets.all(8),
                    child: RaisedChild(Column(
                      children: <Widget>[
                        ListTile(
                          title: Text('Share Video'),
                          leading: Icon(Icons.share),
                          onTap: () {},
                        ),
                        ListTile(
                          title: Text('Quick Send'),
                          leading: Icon(Icons.send),
                          onTap: () {
                            navigate(
                                context, QuickSendVideo(video: widget._video));
                          },
                        ),
                        Container(
                          child: StreamBuilder<List<VideoFolder>>(
                              stream: GetIt.instance<DatabaseService>()
                                  .videoFolderDao
                                  .watchArchivedVideoById(
                                      userInfo.id, widget._video.videoId),
                              builder: (context, snapshot) {
                                if (snapshot.hasData &&
                                    snapshot.data.isNotEmpty) {
                                  return ListTile(
                                      title: Text('Unarchive Video'),
                                      leading: Icon(Icons.unarchive,
                                          color: Colors.lightGreen),
                                      onTap: () {
                                        GetIt.instance<DatabaseService>()
                                            .videoFolderDao
                                            .removeItem(userInfo.id,
                                                widget._video.videoId);
                                      });
                                }
                                return ListTile(
                                  title: Text(
                                    'Archive Video',
                                  ),
                                  leading: Icon(
                                    Icons.archive,
                                    color: Colors.red,
                                  ),
                                  onTap: () {
                                    GetIt.instance<DatabaseService>()
                                        .videoFolderDao
                                        .archivedVid(userInfo.id,
                                            widget._video.videoId, true);
                                    print('pass');
                                  },
                                );
                              }),
                        )
                      ],
                    )),
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  Builder _buildMore(DateFormat formatter, TextEditingController name,
      TextEditingController desc, FocusNode nameNode, FocusNode descNode,
      {bool edit}) {
    return Builder(builder: (_) {
      if (!edit) {
        return Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Divider(),
            Container(
              child: Text(
                widget._video.description,
                softWrap: true,
              ),
            ),
            ListTile(
              leading: Icon(Icons.timer),
              title: Text('Duration'),
              subtitle: Text('${widget._video.duration} s'),
            ),
            ListTile(
              leading: Icon(Icons.desktop_mac),
              title: Text('Resolution'),
              subtitle:
                  Text('${widget._video.width} x ${widget._video.height}'),
            ),
            ListTile(
                leading: Icon(Icons.calendar_today),
                title: Text('Last Modified'),
                subtitle: Text(formatter.format(widget._video.lastUpdated)))
          ],
        );
      } else {
        return Column(
          children: <Widget>[
            AppInputField(
              multiLine: false,
              focus: nameNode,
              name: 'Name',
              controller: name,
              //subtitle: Text(_video.name),
            ),
            AppInputField(
              multiLine: true,
              focus: descNode,
              name: 'Description',
              controller: desc,
            ),
            Padding(
              padding: EdgeInsets.only(top: 10),
              child: Column(
                children: <Widget>[
                  Image.network(
                    widget._video.thumbnail,
                    scale: 0.5,
                  ),
                  Card(
                    child: FlatButton(
                      onPressed: () {},
                      child: Text('Edit Thumbnail'),
                    ),
                  )
                ],
              ),
            )
          ],
        );
      }
    });
  }
}
