import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../common/manage_groups.dart';
import '../../contacts/common/search.dart';
import '../../leads/common/index.dart';
import '../../leads/common/search.dart';

class ContactLeadEmailSelect extends StatefulWidget {
  const ContactLeadEmailSelect({
    Key key,
    @required this.contactsChanged,
    @required this.leadsChanged,
    @required this.contactGroupSelected,
    @required this.leadGroupSelected,
    @required this.emailsChanged,
    @required this.contacts,
    @required this.leads,
  }) : super(key: key);
  final ValueChanged<List<Contact>> contactsChanged;
  final ValueChanged<List<Lead>> leadsChanged;
  final ValueChanged<List<String>> emailsChanged;
  final ValueChanged<List<api.ContactGroup>> contactGroupSelected;
  final ValueChanged<List<api.LeadGroup>> leadGroupSelected;
  final List<Contact> contacts;
  final List<Lead> leads;

  @override
  _ContactLeadEmailSelectState createState() => _ContactLeadEmailSelectState();
}

class _ContactLeadEmailSelectState extends State<ContactLeadEmailSelect> {
  final _controller = TextEditingController();

  List<Contact> _contacts;
  List<Lead> _leads;
  List<String> _emails;
  List<api.LeadGroup> lGroup;
  List<api.ContactGroup> cGroup;

  @override
  void initState() {
    _contacts = widget.contacts;
    _leads = widget.leads;
    _emails = [];
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentInfo(context);
    return Column(
      children: <Widget>[
        ExpansionTile(
          title: Text('Contacts'),
          leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () {
              Navigator.of(context)
                  .push<List<Contact>>(MaterialPageRoute(
                builder: (_) => SearchContacts(
                  selected: _contacts,
                ),
              ))
                  .then((items) {
                if (items != null) {
                  if (mounted)
                    setState(() {
                      _contacts = items;
                    });
                  widget.contactsChanged(_contacts);
                }
              });
            },
          ),
          children: <Widget>[
            if (_contacts == null) ...[
              ListTile(
                title: Text('No Contacts Selected'),
              ),
            ] else ...[
              for (var item in _contacts) ...[
                ListTile(
                  leading: Icon(Icons.person),
                  title: Text(item.firstName),
                  subtitle: Text(item.lastActivity),
                  trailing: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          _contacts.remove(item);
                        });
                      widget.contactsChanged(_contacts);
                    },
                  ),
                ),
              ],
            ],
          ],
        ),
        ExpansionTile(
          title: Text('Leads'),
          leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () {
              Navigator.of(context)
                  .push<List<Lead>>(MaterialPageRoute(
                builder: (_) => SearchLeads(
                  selected: _leads,
                ),
              ))
                  .then((items) {
                if (items != null) {
                  if (mounted)
                    setState(() {
                      _leads = items;
                    });
                  widget.leadsChanged(_leads);
                }
              });
            },
          ),
          children: <Widget>[
            if (_leads == null) ...[
              ListTile(
                title: Text('No Leads Selected'),
              ),
            ] else ...[
              for (var item in _leads) ...[
                ListTile(
                  leading: Icon(Icons.person),
                  title: Text(item.firstName),
                  subtitle: Text(item.lastActivity),
                  trailing: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          _leads.remove(item);
                        });
                      widget.leadsChanged(_leads);
                    },
                  ),
                ),
              ],
            ],
          ],
        ),
        ExpansionTile(
          title: Text('Contact Groups'),
          leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () {
              Navigator.of(context)
                  .push<List<api.ContactGroup>>(MaterialPageRoute(
                      builder: (_) => ManageGroups(
                          type: 'Contact', returnList: true, userInfo: user)))
                  .then((items) {
                if (items != null) {
                  if (mounted) {
                    setState(() {
                      cGroup = items;
                    });
                    widget.contactGroupSelected(cGroup);
                  }
                }
              });
            },
          ),
          children: <Widget>[
            if (cGroup == null) ...[
              ListTile(
                title: Text('No Groups Selected'),
              ),
            ] else ...[
              for (var item in cGroup) ...[
                ListTile(
                  leading: Icon(Icons.people),
                  title: Text(item.name),
                  subtitle: Text(item.count.toString()),
                  trailing: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          cGroup.remove(item);
                        });
                      widget.contactGroupSelected(cGroup);
                    },
                  ),
                ),
              ],
            ],
          ],
        ),
        ExpansionTile(
          title: Text('Lead Groups'),
          leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () {
              Navigator.of(context)
                  .push<List<api.LeadGroup>>(MaterialPageRoute(
                      builder: (_) => ManageGroups(
                          returnList: true, type: 'Lead', userInfo: user)))
                  .then((items) {
                if (items != null) {
                  if (mounted) {
                    setState(() {
                      lGroup = items;
                    });
                    widget.leadGroupSelected(lGroup);
                  }
                }
              });
            },
          ),
          children: <Widget>[
            if (lGroup == null) ...[
              ListTile(
                title: Text('No Groups Selected'),
              ),
            ] else ...[
              for (var item in lGroup) ...[
                ListTile(
                  leading: Icon(Icons.people),
                  title: Text(item.name),
                  subtitle: Text(item.count.toString()),
                  trailing: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      if (mounted)
                        setState(() {
                          lGroup.remove(item);
                        });
                      widget.leadGroupSelected(lGroup);
                    },
                  ),
                ),
              ],
            ],
          ],
        ),
        ExpansionTile(
          leading: IconButton(
            icon: Icon(Icons.email),
          ),
          title: Text('Custom Email'),
          children: <Widget>[
            TextField(
              controller: _controller,
              decoration: InputDecoration(
                hintText: 'add email..',
                border: InputBorder.none,
                filled: true,
                suffixIcon: IconButton(
                  icon: Icon(Icons.check),
                  onPressed: () {
                    setState(() {
                      if(_controller.text.isNotEmpty)
                      _emails.add(_controller.value.text);
                      _controller.text = '';
                    });
                    widget.emailsChanged(_emails);
                  },
                )
              ),
            ),
            if (_emails == null || _emails.isEmpty) ...[
              ListTile(
                title: Text('No Emails Entered'),
              )
            ] else ...[
              for (var email in _emails)
                ...[
                  ListTile(
                    leading: Icon(Icons.alternate_email),
                    title: Text(email),
                    trailing: IconButton(
                      icon: Icon(Icons.close),
                      onPressed: () {
                        if (mounted)
                          setState(() {
                            _emails.remove(email);
                          });
                        widget.emailsChanged(_emails);
                      },
                    ),
                  )
                ]
            ]
          ],
        )
      ],
    );
  }
}
