library unify.impl.io;

import 'dart:async';
import 'dart:convert';

import 'package:http/http.dart' as http;

import '../../unify.dart';
import 'common/http.dart';

class UnifyClientImpl extends UnifyHttpClient {
  UnifyClientImpl(String email, String password, UnifyAccessToken token,
      UnifyApiEndpoints endpoints)
      : super(email, password, token, endpoints);
  var client = new http.Client();
  @override
  Future<dynamic> sendHttpRequest(
    String url, {
    bool needsToken: true,
    String extract,
    dynamic body,
    String method,
    Uri baseUrl,
  }) async {
    final uri = baseUrl ?? endpoints.userApiUrl.resolve(url);
    Future<http.Response> futureRequest;
    final _headers = <String, String>{};
    if (needsToken) {
      _headers['Authorization'] = 'Bearer ${token.accessToken}';
    }
    _headers['Content-Type'] = 'application/json; charset=utf-8';

    if (method != null) {
      if (method == 'PUT') {
        futureRequest = client.put(uri, body: json.encode(body), headers: _headers);
      }
      if (method == 'DELETE') {
        futureRequest = client.delete(uri, headers: _headers);
      }
      if (method == 'POST') {
        futureRequest = client.post(uri, body: json.encode(body), headers: _headers);
      }
      if (method == 'GET') {
        futureRequest = client.get(uri, headers: _headers);
      }
    } else {
      if (body == null) {
        futureRequest = client.get(uri, headers: _headers);
      } else {
        futureRequest =
            client.post(uri, body: json.encode(body), headers: _headers);
      }
    }

    final response = await futureRequest;

    if (response.statusCode != 200) {
      throw Exception(
          'Failed to perform action. (Status Code: ${response.statusCode}\n${response.body})');
    }

    final content = response.body;
    final result = const JsonDecoder().convert(content);

    if (result is Map) {
      if (extract != null) {
        return result[extract];
      }
    }

    return result;
  }

  @override
  Future<dynamic> postHttp(
    String url, {
    String extract,
    Map<String, dynamic> body,
    Uri baseUrl,
    bool raw = false,
  }) async {
    final uri = baseUrl ?? endpoints.userApiUrl.resolve(url);

    final response = await client.post(
      uri.toString(),
      body: body,
    );

    if (response.statusCode != 200) {
      throw Exception(
          'Failed to perform action. (Status Code: ${response.statusCode})\n${response.body}');
    }

    if (raw) {
      return response.body;
    }

    final result = const JsonDecoder().convert(response.body);

    return result;
  }

  @override
  Future close() async {
    client.close();
  }
}
