part of unify;

class ContactDetails implements UnifyObject {
  ContactDetails(this.client, this.json, this.id);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  final String id;

  String get fullName => "$firstName $lastName".trim();

  String get displayName => "$lastName, $firstName";

  String get firstName => json["First_Name"];

  String get middleName => json["Middle_Name"];

  String get lastName => json["Last_Name"];

  String get suffix => json["Suffix"];

  String get company => json["Company"];

  String get secondaryFirstName => json["Secondary_First_Name"];

  String get secondaryMiddleName => json["Secondary_Middle_Name"];

  String get secondaryLastName => json["Secondary_Last_Name"];

  PhoneNumber get cellPhone => PhoneNumber(client, json["Cell_Phone"]);

  PhoneNumber get homePhone => PhoneNumber(client, json["Home_Phone"]);

  PhoneNumber get officePhone => PhoneNumber(client, json["Office_Phone"]);

  PhoneNumber get faxPhone => PhoneNumber(client, json["Fax_Phone"]);

  PhoneNumber get secondaryCellPhone =>
      PhoneNumber(client, json["Secondary_Cell_Phone"]);

  String get email => json["Email_Address"];

  String get secondaryEmail => json["Secondary_Email_Address"];

  String get loanNumber => json["Loan_Number"];

  int get priority => json["Contact_Priority"];

  String get lastActivity => json["Last_Activity"];

  DateTime get dateCreated => DateTime.tryParse(json["Date_Created"] as String);

  DateTime get dateModified =>
      DateTime.tryParse(json["Date_Modified"] as String);

  PostalAddress get currentAddress => PostalAddress(client, json["Address"]);

  CompanyCategory get companyCategory =>
      CompanyCategory(client, json["Company_Category"]);

  List<ContactGroup> get groups => json["Contact_Groups"] == null
      ? []
      : List?.from(json["Contact_Groups"])
              ?.map((i) => ContactGroup(client, i))
              ?.toList() ??
          [];

  List<PhoneNumber> get numbers {
    List<PhoneNumber> _items = [];
    if (cellPhone != null && cellPhone.raw().isNotEmpty) _items.add(cellPhone);
    if (homePhone != null && homePhone.raw().isNotEmpty) _items.add(homePhone);
    if (officePhone != null && officePhone.raw().isNotEmpty)
      _items.add(officePhone);
    return _items ?? [];
  }

  // List<Timeline> get tracker => json["Contact_Timeline"] == null
  //     ? []
  //     : List?.from(json["Contact_Timeline"])
  //             ?.map((i) => Timeline(client, i))
  //             ?.toList() ??
  //         [];

  // List<ContactNote> get notes => json["Contact_Notes"] == null
  //     ? []
  //     : List?.from(json["Contact_Notes"])
  //             ?.map((i) => ContactNote(client, i))
  //             ?.toList() ??
  //         [];

  // List<Task> get tasks => json["Contact_Tasks"] == null
  //     ? []
  //     : List?.from(json["Contact_Tasks"])
  //             ?.map((i) => Task(client, i))
  //             ?.toList() ??
  //         [];

  // List<LoanHistory> get loans => json["Contact_Loan_History"] == null
  //     ? []
  //     : List?.from(json["Contact_Loan_History"])
  //             ?.map((i) => LoanHistory(client, i))
  //             ?.toList() ??
  //         [];

  // -- Actions --
  Future<ContactDetails> getDetails() {
    return client.getContactDetails(id);
  }

  Future<List<ContactNote>> listNotes() {
    return client.listContactNotes(id);
  }

  Future<List<Timeline>> listTimeline() {
    return client.listContactTimeline(id);
  }

  Future<List<Task>> listTasks({DateTime date}) {
    return client.listTasksForContact(id, date: date);
  }

  Future<List<LoanHistory>> listLoans() {
    return client.listLoanHistory(id);
  }

  Future<List<String>> listResponses() {
    return client.listContactLogResponses();
  }

  Future addNote(String text) {
    return client.addContactNote(id, text);
  }

  Future addLogResponse({@required String action, String comment}) {
    return client.addContactLogResponse(id, action: action, comment: comment);
  }

  Future delete() {
    return client.deleteContact(id);
  }

  /// type: `0 (Task)`, `1 (Appointment)`, `2 (Scheduled Call)`
  ///
  /// alertType: `Email`, `TextMessage`, `Email, TextMessage`, ""
  ///
  /// alertOffset: int for `minutes`
  Future addFollowUp({
    @required String title,
    DateTime date,
    int type = 0,
    String description = "",
    bool dismissOnChange = true,
    String alertType,
    int alertOffset = 0,
    String userNumber,
    String prospectNumber,
  }) {
    return client.addContactFollowUp(
      id,
      title: title,
      date: date,
      type: type,
      description: description,
      dismissOnChange: dismissOnChange,
      alertType: alertType,
      alertOffset: alertOffset,
      userNumber: userNumber,
      prospectNumber: prospectNumber,
    );
  }

  Future edit({
    @required String firstName,
    @required String lastName,
    String middleName,
    String suffix,
    String secondaryFirstName,
    String secondaryMiddleName,
    String secondaryLastName,
    PostalAddress currentAddress,
    PhoneNumber cellPhone,
    PhoneNumber homePhone,
    PhoneNumber officePhone,
    PhoneNumber faxPhone,
    PhoneNumber secondaryCellPhone,
    String email,
    String secondaryEmail,
    String loanNumber,
    int contactPriority,
    CompanyCategory companyCategory,
    List<ContactGroup> groups,
  }) {
    return client.editContact(
      id,
      firstName: firstName,
      lastName: lastName,
      middleName: middleName,
      suffix: suffix,
      secondaryFirstName: secondaryFirstName,
      secondaryMiddleName: secondaryMiddleName,
      secondaryLastName: secondaryLastName,
      currentAddress: currentAddress,
      cellPhone: cellPhone,
      homePhone: homePhone,
      officePhone: officePhone,
      faxPhone: faxPhone,
      secondaryCellPhone: secondaryCellPhone,
      email: email,
      secondaryEmail: secondaryEmail,
      loanNumber: loanNumber,
      contactPriority: contactPriority,
      companyCategory: companyCategory,
      groups: groups,
    );
  }
}
