part of unify;

class Video implements UnifyObject, QueryObject, SortableObject<Video> {
  Video(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["id"];

  String get name => json["name"];

  String get description => json["description"];

  String get thumbnail => json["thumbnailUrl"];

  String getThumbnail({int quality = 100, int width = 300}) =>
      json["thumbnailUrl"] + "/quality/$quality/width/$width/";

  String get video => json["videoUrl"];

  String get download => json["downloadUrl"];

  num get width => json["width"];

  num get height => json["height"];

  Duration get duration => Duration(seconds: json["duration"]);

  num get views => num.tryParse(json["views"].toString());

  num get plays => num.tryParse(json["plays"].toString());

  DateTime get dateCreated =>
      DateTime.fromMillisecondsSinceEpoch(json["createdAt"]);

  DateTime get dateModified =>
      DateTime.fromMillisecondsSinceEpoch(json["updatedAt"]);

  String get displayName => "$name";

  // -- Actions --
  Future<VideoDetails> getDetails() {
    return client.getVideoDetails(id);
  }

  Future<String> share({String template = "MESSAGE"}) async {
    return client.getVideoShareUrl(id, template: template);
  }

  Future<String> preview(
      {String template = "MESSAGE", String name, String body}) async {
    return client.getVideoPreview(id,
        template: template, name: name, body: body);
  }

  Future edit({String name, String description}) {
    return client.editVideoInfo(id, name: name, description: description);
  }

  @override
  bool matchesSearch(String search) {
    if (search == null || search.isEmpty) {
      return true;
    }

    search = search.toLowerCase();

    if (name.toLowerCase().contains(search)) {
      return true;
    }
    if (description.toLowerCase().contains(search)) {
      return true;
    }
    if (video.toLowerCase().contains(search)) {
      return true;
    }
    if (thumbnail.toLowerCase().contains(search)) {
      return true;
    }
    if (description.toLowerCase().contains(search)) {
      return true;
    }
    if (duration.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (views.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (dateModified.toString().toLowerCase().contains(search)) {
      return true;
    }

    return false;
  }

  @override
  List<String> sortFields = [
    "name",
    "description",
    "thumbnail",
    "resolution",
    "views",
    "duration",
    "video",
    "date_created",
    "date_modified",
  ];

  @override
  int compareTo(Video object, String sortField, bool sortAscending) {
    int response = 0;
    Video objectA = sortAscending ? this : object;
    Video objectB = sortAscending ? object : this;

    switch (sortField) {
      case "name":
        response = objectA.name.compareTo(objectB.name);
        break;
      case "description":
        response = objectA.description.compareTo(objectB.description);
        break;

      case "thumbnail":
        response = objectA.thumbnail.compareTo(objectB.thumbnail);
        break;

      case "video":
        response = objectA.video.compareTo(objectB.video);
        break;

      case "resolution":
        response = (objectA.width * objectA.height)
            .toString()
            .compareTo((objectB.width * objectB.height).toString());
        break;

      case "views":
        response = objectA.views.compareTo(objectB.views);
        break;

      case "duration":
        response = objectA.duration.compareTo(objectB.duration);
        break;

      case "date_created":
        response = objectA.dateCreated.compareTo(objectB.dateCreated);
        break;
      case "date_modified":
        response = objectA.dateModified.compareTo(objectB.dateModified);
        break;
    }

    if (response == 0) {
      return objectA.name.compareTo(objectB.name);
    } else {
      return response;
    }
  }
}
